----------------------------------------------------------------------------------------------------
-- (c)2011 Altera Corporation. All rights reserved.
--
-- Altera products are protected under numerous U.S. and foreign patents,
-- maskwork rights, copyrights and other intellectual property laws.
--
-- This reference design file, and your use thereof, is subject to and governed
-- by the terms and conditions of the applicable Altera Reference Design License
-- Agreement (either as signed by you or found at www.altera.com).  By using
-- this reference design file, you indicate your acceptance of such terms and
-- conditions between you and Altera Corporation.  In the event that you do not
-- agree with such terms and conditions, you may not use the reference design
-- file and please promptly destroy any copies you have made.
--
-- This reference design file is being provided on an "as-is�" basis and as an
-- accommodation and therefore all warranties, representations or guarantees of
-- any kind (whether express, implied or statutory) including, without
-- limitation, warranties of merchantability, non-infringement, or fitness for
-- a particular purpose, are specifically disclaimed.  By making this reference
-- design file available, Altera expressly does not recommend, suggest or
-- require that this reference design file be used in combination with any
-- other product not provided by Altera.

----------------------------------------------------------------------------------------------------
-- File          : SuperliteII_Demo.vhd (V2 version)
-- Author		 : Peter Schepers
----------------------------------------------------------------------------------------------------
--
-- SuperliteII_Demo : 		- See documentation for details
----------------------------------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use work.package_delaytype.all; -- package that define "Delay_Array" type
 
entity SuperliteII_Demo is
PORT(
	RefClock						: in std_logic; 						
	Clk100Mhz					: in std_logic;
	Enable_Core					: in std_logic;
	XCVR_TX						: OUT STD_LOGIC_VECTOR (3 DOWNTO 0);	-- 10 Channels
	XCVR_RX						: IN STD_LOGIC_VECTOR (3 DOWNTO 0);     
	led							: out std_logic_vector(7 downto 0);
	Control_Reg					: in std_logic_vector(15 downto 0);
	Control2_Reg				: in std_logic_vector(15 downto 0);
	reconfig_reset          : in  std_logic; 
	reconfig_write          : in  std_logic; 
	reconfig_read           : in  std_logic; 
	reconfig_address        : in  std_logic_vector(15 downto 0)  := (others => '0'); 
	reconfig_writedata      : in  std_logic_vector(31 downto 0)  := (others => '0'); 
	reconfig_readdata       : out std_logic_vector(31 downto 0);   
	reconfig_waitrequest    : out  std_logic;
	reconfig_pll_reset          : in  std_logic; 
	reconfig_pll_write          : in  std_logic; 
	reconfig_pll_read           : in  std_logic; 
	reconfig_pll_address        : in  std_logic_vector(15 downto 0)  := (others => '0'); 
	reconfig_pll_writedata      : in  std_logic_vector(31 downto 0)  := (others => '0'); 
	reconfig_pll_readdata       : out std_logic_vector(31 downto 0);   
	reconfig_pll_waitrequest    : out  std_logic;
	Channel_Reg					: out std_logic_vector(15 DOWNTO 0);
	Counter_1ms_Reg			: out std_logic_vector(31 DOWNTO 0);
	ErrorCount_Reg				: out std_logic_vector(15 DOWNTO 0);
	Bitrate_Reg					: out std_logic_vector(31 DOWNTO 0);
	DataClock_Reg				: out std_logic_vector(31 DOWNTO 0);
	DataOut_Clock_Reg			: out std_logic_vector(31 DOWNTO 0);
	Latency_Max_Reg			: out std_logic_vector(7 downto 0);
	Latency_Min_Reg			: out std_logic_vector(7 downto 0);
	Delay_Reg					: out std_logic_vector(31 downto 0) := (OTHERS => '0');
	Delay_Reg_2					: out std_logic_vector(31 downto 0) := (OTHERS => '0');
	Temperature					: out std_logic_vector(7 downto 0)	
	);
END SuperliteII_Demo;	


architecture rtl of SuperliteII_Demo is

component Reset_Synchro
port
	(
	Clk			: in std_logic;
	Reset_in	: in std_logic;
	Reset_out	: out std_logic
	);
end component;

component CountGenerate 
PORT(
	coreclk  	: in std_logic;
	Enable	: in std_logic;
	StartValue : in std_logic_vector(15 downto 0);	
	Reset	 	: in std_logic;
	Inserterror : in std_logic;
	Counterout 	: out std_logic_vector(15 downto 0);
	Valid			: out std_logic
	);
END component;	

component CountVerify
PORT(
	Clock  				: in std_logic;
	Reset	 			: in std_logic;
	ResetErrorCount 	: in std_logic;
	DataIn 				: in std_logic_vector(15 downto 0);
	Enable				: in std_logic;	
	CountLocked			: out std_logic;
	Errorcount_Q 		: out std_logic_vector(15 downto 0);
	Count_Pattern_Q		: out std_logic_vector(15 downto 0)
	);
END component;		


component PrbsGenerate_60bit 
PORT(
	coreclk  	: in std_logic;
	Enable		: in std_logic;
	StartValue	: in std_logic_vector(59 downto 0);
	Reset	 	: in std_logic;
	Inserterror : in std_logic;
	Prbsout 	: out std_logic_vector(59 downto 0);
	Valid		: out std_logic
	);
END component;	
		

component PrbsVerify_60bit 
PORT(
	RxClock  			: in std_logic;
	Enable				: in std_logic;
	Reset	 			: in std_logic;
	ResetErrorCount 	: in std_logic;
	DataIn 				: in std_logic_vector(59 downto 0);
	PrbsLocked			: out std_logic;
	Errorcount_Q 		: out std_logic_vector(15 downto 0);
	next_prbs_Q			: out std_logic_vector(59 downto 0)
	);
END component;	

--component FEC_Error_Verify is
--PORT(
--	RxClock  						: in std_logic;
--	Reset	 							: in std_logic;
--	ResetErrorCount 				: in std_logic;
--	StatusIn 						: in std_logic_vector(1 downto 0);
--	FEC_Correctable_Lane 		: out std_logic_vector(31 downto 0);
--	FEC_UnCorrectable_Lane		: out std_logic_vector(15 downto 0)
--	);
--END component;	



component superliteii_txrx_module 
GENERIC
	(
		READ_LENGTH			: std_logic_vector(15 downto 0) := X"00FF";
		IDLE_LENGTH			: std_logic_vector(3 downto 0)  := X"7";		
		NUMBER_OF_LANES				: integer := 10;	  	-- NUMBER_OF_LANES
		LANEWIDTH					: integer := 64	-- LANEWIDTH for transceiver
	);
PORT(
	Reset						: in std_logic;	
	RefClock					: in std_logic; 						-- RefClock Transceiver 
	Clk100Mhz				: in std_logic;						-- Required as reconfig clock and calibration clock
	
	-- Tx Path
	DataIn						: in std_logic_vector(((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);
	DataIn_Valid				: in std_logic; 					-- Only write when DataIn_valid is high	
	XOFF							: in std_logic;		-- XOFF input to sent out XOFF to the remote device for the remote to stop sending traffic (100 Mhz Clock domain)
	DataClock					: in std_logic;
	DataIn_ready				: out std_logic;			-- Asserted when FIFO can accept new data;
	
	Reset_Tx						: buffer std_logic;
	TxCoreClock					: buffer std_logic;
	
	
	-- High speed serial data outputs	
	XCVR_TX					: out std_logic_vector((NUMBER_OF_LANES-1) DOWNTO 0); 	  	
	
	-- Status signals	
	pll_locked					: buffer std_logic_vector(0 downto 0);
	wrfull_Tx_Ratematcher		: out std_logic;	
	tx_ready						: buffer std_logic;

   -- Rx Path
	
	ForceAlign					: in std_logic;	-- Force the Rx_Path to re-align
	Reset_Rx						: buffer std_logic;
	
	-- High speed serial input data lanes
	XCVR_RX						: in std_logic_vector((NUMBER_OF_LANES-1) DOWNTO 0); 
   Serial_Loop					: in std_logic_vector((NUMBER_OF_LANES-1) DOWNTO 0); 

	
	-- Parallel Data Output (with Valid signal) and clock
	DataOut						: out std_logic_vector(((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);
	DataOut_Valid				: out std_logic; 
	DataOut_Clock				: buffer std_logic;  
			
	-- Transceiver reconfiguration interface	
	reconfig_reset          : in  std_logic; 
	reconfig_write          : in  std_logic; 
	reconfig_read           : in  std_logic; 
	reconfig_address        : in  std_logic_vector(15 downto 0)  := (others => '0'); 
	reconfig_writedata      : in  std_logic_vector(31 downto 0)  := (others => '0'); 
	reconfig_readdata       : out std_logic_vector(31 downto 0);   
	reconfig_waitrequest    : out  std_logic;
	-- Tx PLL reconfiguration interface	
	reconfig_pll_reset          : in  std_logic; 
	reconfig_pll_write          : in  std_logic; 
	reconfig_pll_read           : in  std_logic; 
	reconfig_pll_address        : in  std_logic_vector(15 downto 0)  := (others => '0'); 
	reconfig_pll_writedata      : in  std_logic_vector(31 downto 0)  := (others => '0'); 
	reconfig_pll_readdata       : out std_logic_vector(31 downto 0);   
	reconfig_pll_waitrequest    : out  std_logic;

  
	-- Status signals	
	rx_freqlocked				: buffer std_logic_vector((NUMBER_OF_LANES-1) DOWNTO 0); 	
   rx_ready						: buffer std_logic;	
	Reset_Rx_Path				: buffer std_logic;
	WordAligned					: buffer std_logic;
	LaneAligned					: buffer std_logic;
	Linkup						: buffer std_logic;
	XOFF_Received				: buffer std_logic;	
	Rx_Error						: buffer std_logic;	
	decoder_error				: out std_logic;	
	Error_Deskew				: out std_logic;
	--rx_enh_frame_diag_status : buffer std_logic_vector(((NUMBER_OF_LANES*2)-1) DOWNTO 0); 
	Delay							: buffer Delay_Array			
	);
END component;


--	component pll_data is
--		port (
--			locked   : out std_logic;        -- export
--			outclk_0 : out std_logic;        -- clk
--			refclk   : in  std_logic := 'X'; -- clk
--			rst      : in  std_logic := 'X'  -- reset
--		);
--	end component pll_data;

	


component intnoise 
port
	(
	clock	: in	std_logic;
	enable	: in	std_logic;
	q		: out	std_logic
	);
end component;

component alt_a10_temp_sense 
port
(
	clk : in std_logic;
	degrees_c : out std_logic_vector(7 downto 0);
	degrees_f : out std_logic_vector(7 downto 0)
);
end component;




CONSTANT CLK50M				: std_logic_vector(19 downto 0) := X"0C350" ; 
CONSTANT CLK100M				: std_logic_vector(19 downto 0) := X"186A0" ; --

CONSTANT SAMPLES_50MHZ			: std_logic_vector(31 downto 0) :=	X"02FAF080"; -- (50 Mhz clock is 50E6 samples in one second)
CONSTANT SAMPLES_50MHZ_TIMES_5	: std_logic_vector(31 downto 0) :=	X"0EE6B280"; 
CONSTANT SAMPLES_100MHZ				: std_logic_vector(31 downto 0) :=	X"05F5E100"; -- (50 Mhz clock is 50E6 samples in one second)
CONSTANT SAMPLES_100MHZ_TIMES_5	: std_logic_vector(31 downto 0) :=	X"1DCD6500"; 


component measure_refclk 
GENERIC
	(
		CYC_MEASURE_CLK_IN_1_SEC	:  std_logic_vector(31 downto 0) := SAMPLES_50MHZ 
	);
port
	(
	RefClock				: in	std_logic;
	Enable				: in std_logic;
	Measure_Clk				: in 	std_logic;
	reset					: in	std_logic;
	RefClock_Measure		: out  std_logic_vector(31 downto 0) 
	);
end component;

component counter_1ms 
GENERIC
	(
		BITRATE	: std_logic_vector(19 downto 0) :=	CLK50M
	);
port
	(
	RefClock		: in	std_logic;
	reset			: in	std_logic;
	count_1ms		: buffer std_logic_vector(31 downto 0)
	);
end component;


CONSTANT NUMBER_OF_LANES	: integer := 4; --Number of links
CONSTANT NUMBER_OF_QUADS	: integer := 1; --Number of Quads
CONSTANT LANEWIDTH 			: integer := 64;
CONSTANT REFCLOCKMULTIPLIER : std_logic_vector(5 downto 0) := "010000"; -- Multiplier of the Reference clock to reach bitrate per lane (16)

type Bit60Type is array (0 to (NUMBER_OF_LANES-1)) of std_logic_vector(59 downto 0);
type Bit16Type is array (0 to (NUMBER_OF_LANES-1)) of std_logic_vector(15 downto 0);
type Bit32Type is array (0 to (NUMBER_OF_LANES-1)) of std_logic_vector(31 downto 0);

signal TxData 				: std_logic_vector(((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);

signal Reset 				: std_logic;
signal ResetErrorCount		: std_logic;

signal inserterror 			: std_logic;
signal CountLocked 		: std_logic;
signal PrbsLocked 		: std_logic_vector((NUMBER_OF_LANES - 1) downto 0);


signal Locked 				: std_logic;

signal pll_locked 			: std_logic_vector(0 downto 0);

signal Data_Out_Rx			: std_logic_vector(((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);

signal DataOut_Aligned 		: std_logic_vector(((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);
signal Ctrl_Aligned			: std_logic_vector((NUMBER_OF_LANES - 1) downto 0);
signal Byte_Aligned			: std_logic_vector((NUMBER_OF_LANES - 1) downto 0);
signal Word_Aligned			: std_logic_vector((NUMBER_OF_LANES - 1) downto 0);

signal ErrorCount 					: std_logic_vector(15 downto 0);	
signal ErrorCount_CountPattern 	: std_logic_vector(15 downto 0);
signal Errorcount_PrbsPattern	 	: Bit16Type;

signal CounterOut					: std_logic_vector(((NUMBER_OF_LANES * 4)-1) downto 0);
signal Prbsout						: Bit60Type;
signal DataIn_Tx				: std_logic_vector(((NUMBER_OF_LANES * LANEWIDTH)-1) downto 0);
signal ResetErrorCountChannel	: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal PowerOnReset 			: std_logic;
signal ResetCount				: std_logic_vector(5 downto 0) := (OTHERS => '0');
signal intnoise_out 			: std_logic;
signal count_pattern_Q 			: std_logic_vector(15 downto 0);
signal next_prbs_Q				: std_logic_vector(31 downto 0);
signal ChannelOk				: std_logic;
signal ChannelOk_Q				: std_logic;
signal rx_freqlocked			: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal rx_freqlocked_Q		: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal rx_freqlocked_Q1		: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal rx_freqlocked_Q2		: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal rx_freqlocked_combined	: std_logic;
signal Reset_I					: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal Count					: std_logic;
signal Enable_Rx				: std_logic;
signal Aligned					: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);

signal Reset_Counter_1ms		: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal Reset_CountGenerate		: std_logic_vector ((NUMBER_OF_LANES - 1) downto 0);
signal Reset_DataVerify			: std_logic;

signal DataClock				: STD_LOGIC;
signal DataOut_Clock			: std_logic;
signal data_valid				: STD_LOGIC ;
signal RefClock_Measure		: std_logic_vector(31 downto 0);
signal DataClock_Measure		: std_logic_vector(31 downto 0);
signal DataOut_Clock_Measure		: std_logic_vector(31 downto 0);
signal ResetCounter_1ms			: std_logic;
signal ClkData				: std_logic;
signal locked_pll				: std_logic;
signal Reset_Tx					: std_logic;
signal Reset_Rx					: std_logic;
signal TxCoreClock				: std_logic;
signal RxCoreClock				: std_logic;
signal wrfull_Tx_Ratematcher	: std_logic;
signal ResetErrorCount_I		: std_logic;
signal WordAligned				: std_logic;
signal LaneAligned				: std_logic;
signal Reset_Rx_Path			: std_logic;
signal Reset_DataGenerate		: std_logic;
signal Reset_TxRxModule			: std_logic;
signal LaneAlignment_Changed	: std_logic_vector((NUMBER_OF_LANES-1) DOWNTO 0);
signal LaneAlignment_Changed_All: std_logic;

signal Force_Realign			: std_logic;

signal PrbsDataIn					: Bit60Type;
signal StartValue					: Bit60Type;
signal CountVerifyDataIn			: std_logic_vector(((NUMBER_OF_LANES * 4)-1) downto 0);
signal Serial_Loop				: std_logic_vector((NUMBER_OF_LANES-1) DOWNTO 0);
signal tx_clkout_ALTGX			: std_logic_vector((NUMBER_OF_LANES-1) DOWNTO 0);
signal locked_pll_Q				: std_logic;
signal locked_pll_Q1				: std_logic;
signal locked_pll_Q2				: std_logic;
signal locked_pll_R				: std_logic;
signal locked_pll_R1				: std_logic;
signal locked_pll_R2			: std_logic;
signal pll_locked_Q				: std_logic;
signal pll_locked_Q1				: std_logic;
signal pll_locked_Q2			: std_logic;
signal pll_locked_R				: std_logic;
signal pll_locked_R1				: std_logic;
signal pll_locked_R2			: std_logic;
signal decoder_error			: std_logic;

signal Debug_Measure			: std_logic_vector(31 downto 0);

signal ForceAlign				: std_logic;
signal RefClockdiv2			: std_logic;
signal Ones 					: std_logic_vector((NUMBER_OF_LANES-1) DOWNTO 0); 
signal busy						: std_logic;

		
signal reconfig_reset_poweron  : std_logic;	
signal reconfig_reset_i			 : std_logic;
signal reconfig_pll_reset_i		 : std_logic;


signal Synchro 				: std_logic_vector(2 downto 0);
signal Synchro2 				: std_logic_vector(63 downto 0);
signal Synchro3 				: std_logic_vector(2 downto 0);
signal Synchro4 				: std_logic_vector(2 downto 0);
signal Error_Deskew 			: std_logic;
signal Delay					: Delay_Array;
signal Error_pcfifo			: std_logic;
signal Measure_diff 			: std_logic_vector(((NUMBER_OF_LANES * 4)-1) downto 0);
signal Simulation				: std_logic;
signal DataIn_Valid			: std_logic;
signal DataIn_Valid_min1	: std_logic;
signal Valid_Counter			: std_logic_vector(7 downto 0);
signal Reset_in1		 		: std_logic;
signal rx_enh_frame_diag_status : std_logic_vector(((NUMBER_OF_LANES * 2)-1) downto 0);
signal FEC_Correctable_Lane	: Bit32Type;
signal FEC_UnCorrectable_Lane : Bit16Type;
signal correctable_errorcount : std_logic_vector(31 downto 0);
signal uncorrectable_errorcount : std_logic_vector(15 downto 0);

signal DataIn_ready				: std_logic;

signal Latency_Max			: std_logic_vector(((NUMBER_OF_LANES * 2)-1) downto 0);
signal Latency_Min			: std_logic_vector(((NUMBER_OF_LANES * 2)-1) downto 0);
signal Measure_diff_Q		: std_logic_vector(((NUMBER_OF_LANES * 2)-1) downto 0);
signal count_latency					: std_logic_vector(6 downto 0);
signal count_no_match		: std_logic_vector(6 downto 0);
signal no_match				: std_logic;
signal Reset_in3				: std_logic;

signal Linkup					: std_logic;
signal XOFF						: std_logic;
signal XOFF_Received			: std_logic;
signal Rx_Error				: std_logic;
signal CounterOut_min2		: std_logic_vector(((NUMBER_OF_LANES * 4)-1) downto 0);
signal CounterOut_min1		: std_logic_vector(((NUMBER_OF_LANES * 4)-1) downto 0);
signal CounterOut_Q			: std_logic_vector(((NUMBER_OF_LANES * 4)-1) downto 0);

begin

-- Reset : Control_Reg(15)
-- 1 : Reset
-- 0 : No Reset

-- InsertError : Control_Reg(14)
-- 1 : InsertError
-- 0 : NoError

-- ResetErrorCount : Control_Reg(13)
-- 1 : ResetErrorcount active
-- 0 : No Reset

-- Force_Realign : Control_Reg(12)
-- 1 : Force Realign
-- 0 : Nothing

-- XOFF : Control_Reg(11)
-- 1 : Sent out XOFF to the remote side to stop sending traffic
-- 0 : XON.

 
-- Serial Loop : Control2_Reg(3..0)
-- 1 : Serial Loopback enabled
-- 0 : No action




----------------------------------------------------------------------------------------
-- PowerOnReset
----------------------------------------------------------------------------------------


process(Clk100Mhz)
begin
	if Clk100Mhz'event and Clk100Mhz = '1' then
			if ResetCount = "111111" then
				PowerOnReset <= '0';
			else
				PowerOnReset <= '1';
			  	ResetCount <= ResetCount + 1;
			end if;
	end if;
end process;

----------------------------------------------------------------------------------------
-- Control bits
----------------------------------------------------------------------------------------

inserterror			<= Control_Reg(14);
ResetErrorCount 	<= Control_Reg(13);
Force_Realign		<= Control_Reg(12);
XOFF					<= Control_Reg(11);
Serial_Loop			<= Control2_Reg((NUMBER_OF_LANES-1) downto 0); 


----------------------------------------------------------------------------------------
-- Generate Data clock frequency for data generator (180 Mhz)
----------------------------------------------------------------------------------------

--pll_data_inst : pll_data PORT MAP (
--		refclk 		=> Clk100Mhz,
--		rst			=> PowerOnReset,
--		outclk_0	 			=> ClkData,
--		locked	 		=> locked_pll
--	);
--	


----------------------------------------------------------------------------------------
-- Data Clock is the clock from the transceiver
----------------------------------------------------------------------------------------

ClkData 		<= TxCoreClock;
--locked_pll 	<= pll_locked(0);
	

----------------------------------------------------------------------------------------
-- Create Reset
----------------------------------------------------------------------------------------

process(Clk100Mhz,PowerOnReset)
begin
	if PowerOnReset = '1' then
		Reset 				<= '1';
		reconfig_reset_poweron 	<= '1';			
	elsif Clk100Mhz'event and Clk100Mhz = '1' then

		if (Enable_Core = '1') and (Control_Reg(15) = '0')	then	
			Reset	 			<= '0';
			reconfig_reset_poweron	<= '0';				
		else
			Reset 			<= '1';
		end if;
	end if;
end process;

----------------------------------------------------------------------------------------
-- Create reconfig_reset such it is de-asserted at the same time as Reset (Which is used for the reset controller).
----------------------------------------------------------------------------------------

reconfig_reset_i 	<= reconfig_reset or reconfig_reset_poweron;
reconfig_pll_reset_i 	<= reconfig_pll_reset or reconfig_reset_poweron;


----------------------------------------------------------------------------------------
-- Create Reset_DataGenerate
----------------------------------------------------------------------------------------

process(ClkData,Reset)
begin
	if Reset = '1' then
		Reset_DataGenerate	 <= '1';
		pll_locked_R		 <= '0';
		pll_locked_R1		 <= '0';
		pll_locked_R2		 <= '0';		
	elsif rising_edge(ClkData) then
	
		pll_locked_R			<= pll_locked(0);
		pll_locked_R1			<= pll_locked_R;	
	   pll_locked_R2			<= pll_locked_R1;	
		if (pll_locked_R2 = '1')  then
			Reset_DataGenerate	 		<= '0';
		else
			Reset_DataGenerate 			<= '1';
		end if;
	end if;
end process;

----------------------------------------------------------------------------------------
-- DataGenerators 
----------------------------------------------------------------------------------------

	
CountGenerate_inst:CountGenerate PORT MAP(
	coreclk  		=> ClkData,
	Reset	 			=> Reset_DataGenerate,
	StartValue		=> (OTHERS => '0'),
	Enable			=> DataIn_ready,
	Inserterror 	=> '0', -- The counterpattern is used only to make sure the lanes are aligned.
	Counterout 		=> Counterout, 
	Valid				=> DataIn_valid
	);	

StartValue(0)		<= X"012345677654321";
StartValue(1)		<= X"111111111111111";
StartValue(2)		<= X"222222222222222";
StartValue(3)		<= X"333333333333333";

	

Generate_PrbsGenerate:
FOR I IN 0 to NUMBER_OF_LANES-1 GENERATE	
PrbsGenerate_inst:PrbsGenerate_60bit PORT MAP(
	coreclk  		=> ClkData,
	Reset	 			=> Reset_DataGenerate,
	StartValue		=> StartValue(I),
	Enable			=> DataIn_ready,
	Inserterror 	=> InsertError,
	Prbsout 			=> Prbsout(I),
	Valid				=> open -- Should be synchronously with the counterdata
	);	
END GENERATE;

----------------------------------------------------------------------------------------
-- Map Counter and Prbs patterns to the 256 bit bus for the 4 lanes
-- Use 4 bits of counterpattern for 4 MSB (spread over the 4 lanes) and 60 lower bits for a PRBS-23 pattern
----------------------------------------------------------------------------------------

DataIn_Tx(63 downto 0) 		<=  CounterOut(3  downto 0)   & Prbsout(0);
DataIn_Tx(127 downto 64) 	<=  CounterOut(7  downto 4)   & Prbsout(1);
DataIn_Tx(191 downto 128) 	<=  CounterOut(11  downto 8)   & Prbsout(2);
DataIn_Tx(255 downto 192) 	<=  CounterOut(15  downto 12)   & Prbsout(3);



----------------------------------------------------------------------------------------
-- SuperliteII TXRX Module
----------------------------------------------------------------------------------------


superliteii_txrx_module_inst: superliteii_txrx_module 
GENERIC MAP
	(
		READ_LENGTH								=> X"00FF",
		IDLE_LENGTH								=> X"7",	
		NUMBER_OF_LANES		=>  NUMBER_OF_LANES,	  	-- NUMBER_OF_LANES
		LANEWIDTH				=> LANEWIDTH		-- LANEWIDTH for transceiver
	)
PORT MAP(
	Reset						=> Reset,
	
	RefClock					=> RefClock, 	
--	Disable_Rx				=> Disable_Rx,
--	Disable_Tx				=> Disable_Tx,
	
	Serial_Loop				=> Serial_Loop,
	Clk100Mhz 				=> Clk100Mhz,	
			
		
	-- Parallel Data input clocked with DataClock (coming from the transceiver)
	DataIn						=> DataIn_Tx,
	DataIn_Valid				=> DataIn_Valid, 
	DataClock					=> ClkData,	
	DataIn_ready				=> DataIn_ready,
	XOFF							=> XOFF,	
	TxCoreClock					=> TxCoreClock,  -- for debug
	
	-- High speed serial data outputs
	XCVR_TX						=> XCVR_TX,
	
	
	-- Status signals	
	pll_locked					=> pll_locked,
	wrfull_Tx_Ratematcher	=> wrfull_Tx_Ratematcher,	
	tx_ready						=> open,
	
	ForceAlign					=> ForceAlign,			
	Reset_Rx						=> Reset_Rx,
	
	-- High speed serial input data lanes
	XCVR_RX						=> XCVR_RX, 
	
	-- Parallel Data Output (with Valid signal) and clock
	DataOut						=> Data_Out_Rx,
	DataOut_Valid				=> Enable_Rx,
	DataOut_Clock				=> DataOut_Clock, 	

			
	-- Status signals	
	rx_freqlocked				=> rx_freqlocked,
	rx_ready						=> open,
	Reset_Rx_Path				=> Reset_Rx_Path,
	WordAligned					=> WordAligned,
	decoder_error				=> decoder_error,
	Error_Deskew				=> Error_Deskew,	
	LaneAligned					=> LaneAligned,
	Linkup						=> Linkup,
	XOFF_Received				=> XOFF_Received,	
	Rx_Error						=> Rx_Error,
	--rx_enh_frame_diag_status => open,	
	Delay							=> Delay,

	-- Transceiver reconfiguration interface	
		reconfig_reset		      => reconfig_reset_i,
		reconfig_write          => reconfig_write,
		reconfig_read           => reconfig_read,
		reconfig_address        => reconfig_address,
		reconfig_writedata      => reconfig_writedata,
		reconfig_readdata       => reconfig_readdata,
		reconfig_waitrequest    => reconfig_waitrequest,
	
	-- LC PLL reconfiguration interface	
	
		reconfig_pll_reset       	=> reconfig_pll_reset_i,
		reconfig_pll_write       	=> reconfig_pll_write,
		reconfig_pll_read        	=> reconfig_pll_read,
		reconfig_pll_address        => reconfig_pll_address,
		reconfig_pll_writedata      => reconfig_pll_writedata,
		reconfig_pll_readdata       => reconfig_pll_readdata,
		reconfig_pll_waitrequest 	=> reconfig_pll_waitrequest
	
	);
	

	
----------------------------------------------------------------------------------------
-- Synchronize Force_Realign input to DataOut_Clock
----------------------------------------------------------------------------------------

						
process(DataOut_Clock,Reset_Rx_Path)
begin
	if (Reset_Rx_Path = '1') then
		Synchro4			<= "000";
		ForceAlign		<= '0';
	elsif rising_edge(DataOut_Clock) then
		Synchro4		<= Synchro4(1 downto 0) & (Force_Realign);
		ForceAlign	<= Synchro4(2);
	end if;
end process;	

-----------------------------------------------------------------------------------------------------------
--	Synchronize to Clk100Mhz
-----------------------------------------------------------------------------------------------------------	

process(Clk100Mhz,Reset)	
begin
	if Reset = '1' then 					
			rx_freqlocked_Q 	<= (OTHERS => '0');
			rx_freqlocked_Q1 	<= (OTHERS => '0'); 
			rx_freqlocked_Q2 	<= (OTHERS => '0');			

			pll_locked_Q 		<=  '0';	
			pll_locked_Q1 		<=  '0';	
			pll_locked_Q2 		<=  '0';	
	elsif rising_edge(Clk100Mhz) then	
--	
		rx_freqlocked_Q <= rx_freqlocked;
		rx_freqlocked_Q1 <= rx_freqlocked_Q;
		rx_freqlocked_Q2 <= rx_freqlocked_Q1;  -- This is to synchronize Rx_Freqlocked to the phy_mgmt_clk domain
		
		pll_locked_Q	 <= pll_locked(0);
		pll_locked_Q1	 <= pll_locked_Q;		-- This is to synchronize Pll_Locked to the phy_mgmt_clk domain
		pll_locked_Q2	 <= pll_locked_Q1;
		
	end if;
end process;	


-----------------------------------------------------------------------------------------------------------
--	Counter Verification Instantiation
-----------------------------------------------------------------------------------------------------------	

process(DataOut_Clock,Reset_Rx_Path)
begin
	if Reset_Rx_Path = '1' then
		Reset_DataVerify  <= '1';
	elsif rising_edge(DataOut_Clock) then
		if (LaneAligned = '1') then
			Reset_DataVerify	 <= '0';
		else
			Reset_DataVerify 	<= '1';
		end if;
	end if;
end process;

-----------------------------------------------------------------------------------------------------------
--	Synchronize ResetErrorCount
-----------------------------------------------------------------------------------------------------------	

Reset_in1	<= Reset or ResetErrorCount; -- Both are generated on Clk100Mhz

Reset_Synchro_inst1 : Reset_Synchro
PORT MAP
	(
	Clk			=> DataOut_Clock,
	Reset_in		=> Reset_in1,
	Reset_out	=> ResetErrorCount_I 
	);
	


CountVerifyDataIn 	 <=  Data_Out_Rx(255 downto 252) & Data_Out_Rx(191 downto 188) & Data_Out_Rx(127 downto 124) & Data_Out_Rx(63 downto 60); 

PrbsDataIn(0) 	   <= Data_Out_Rx(59 downto 0);
PrbsDataIn(1) 	   <= Data_Out_Rx(123 downto 64);
PrbsDataIn(2) 	   <= Data_Out_Rx(187 downto 128);
PrbsDataIn(3) 		<= Data_Out_Rx(251 downto 192);



CountVerify_inst:CountVerify PORT MAP(
	Clock  				=> DataOut_Clock,
	Reset	 				=> Reset_DataVerify,
	ResetErrorCount 	=> ResetErrorCount_I,
	DataIn				=> CountVerifyDataIn,	
	Enable				=> Enable_Rx,
	CountLocked			=> CountLocked,
	ErrorCount_Q  		=> Errorcount_CountPattern
	--count_pattern_Q		=> count_pattern_Q
	);	
		
Generate_PrbsVerify:
FOR I IN 0 to NUMBER_OF_LANES-1 GENERATE	
PrbsVerify_instX:PrbsVerify_60bit PORT MAP(
	RxClock  			=> DataOut_Clock,
	Reset	 				=> Reset_DataVerify,
	ResetErrorCount 	=> ResetErrorCount_I,
	DataIn				=> PrbsDataIn(I),	
	Enable				=> Enable_Rx,
	PrbsLocked			=> PrbsLocked(I),
	ErrorCount_Q  		=> Errorcount_PrbsPattern(I)
	);
END GENERATE;

--Generate_FEC_ErrorCount:
--FOR I IN 0 to NUMBER_OF_LANES-1 GENERATE	
--FEC_Error_Verify_instX:FEC_Error_Verify PORT MAP(
--	RxClock  							=> DataOut_Clock,
--	Reset	 								=> Reset_DataVerify,
--	ResetErrorCount 					=> ResetErrorCount_I,
--	StatusIn								=> rx_enh_frame_diag_status((2*(I+1)-1) downto (2*I)) ,	
--	FEC_Correctable_Lane  			=> FEC_Correctable_Lane(I),
--	FEC_UnCorrectable_Lane  		=> FEC_UnCorrectable_Lane(I)
--	);
--END GENERATE;




-----------------------------------------------------------------------------------------------------------
--	Measure Maximum and Minimum Latency 
-- Note that this measurement can occasionally occur in reading out 0 for Maximum Latency (depending on the 
-- phases of the clock) (and similar 256 for Minimum latency). These values should be discarded.
-----------------------------------------------------------------------------------------------------------	

process(DataOut_Clock,Reset_Rx_Path)

begin
	if Reset_Rx_Path = '1' then
			Measure_diff  	 <= (OTHERS => '0');
			Measure_diff_Q  <= (OTHERS => '0');
			Latency_Max <= (OTHERS => '0');
			Latency_Min <= (OTHERS => '1');
			no_match <= '0';
			count_latency <= (OTHERS => '0');
			count_no_match <= (OTHERS => '0');
			CounterOut_min2	<= (OTHERS => '0');
			CounterOut_min1	<= (OTHERS => '0');
			CounterOut_Q	<= (OTHERS => '0');			
			
	elsif rising_edge(DataOut_Clock) then
			if CountLocked = '1' then
				if  Enable_Rx = '1' then
				
				
				   CounterOut_min2 <= CounterOut;
					Counterout_min1 <= CounterOut_min2;
					CounterOut_Q	 <= CounterOut_min1;   -- Synchronize CounterOut to DataOut_Clock domain.
					
					
					Measure_diff <=  Counterout_Q - countverifyDataIn;
					--Latency_Reg	<= Measure_diff(7 downto 0);

				
					Measure_diff_Q <= Measure_diff(7 downto 0);

					if (Measure_diff_Q /= Measure_diff(7 downto 0))  then
						no_match <= '1';
					else
						no_match <= '0';
					end if;
					
					if no_match = '1' then
						count_latency <= (Others => '0');
						count_no_match <= count_no_match + 1;
					else
						count_latency <= count_latency + 1;
						count_no_match <=(Others => '0');
					end if;
					if count_latency(5) = '1' and count_no_match = 0 then -- Receive 32 times in a row the same latency.
						if  Measure_diff(7 downto 0) > Latency_Max then
							Latency_Max <= Measure_diff_Q+3; -- +3 to incorporate for the additional delay of the retiming
						end if;
						if  Measure_diff(7 downto 0)  < Latency_Min then
							Latency_Min  <= Measure_diff_Q+3 ; -- +3 to incorporate for the additional delay of the retiming
						end if;
					end if;


				end if;
				
			end if;
	end if;
end process;


-----------------------------------------------------------------------------------------------------------
--	Internal Noise Generator
-----------------------------------------------------------------------------------------------------------	

Intnoise_inst:intnoise PORT MAP(
	clock => RefClock,
	enable => '0',
	q	=> intnoise_out
	);

-----------------------------------------------------------------------------------------------------------
--	1ms Counter Instantiation
-----------------------------------------------------------------------------------------------------------	

Counter_1ms_inst:counter_1ms 
GENERIC MAP
	(
		BITRATE	 =>	CLK100M  
	)
PORT MAP(
	RefClock 	=> Clk100Mhz,
	Reset 		=> Reset_in1,
	count_1ms	=> Counter_1ms_Reg -- every tick of the counter corresponds to 1ms/
	);

-----------------------------------------------------------------------------------------------------------
--	Measure RefClock 
-----------------------------------------------------------------------------------------------------------	

measure_refclk_inst : measure_refclk
GENERIC MAP
	(
		CYC_MEASURE_CLK_IN_1_SEC	=> SAMPLES_100MHZ
	)
port MAP(
	RefClock					=> RefClock,
	Enable					=> '1',	
	Measure_Clk				=> Clk100Mhz,
	reset						=> PowerOnReset,
	RefClock_Measure		=> RefClock_Measure
	);


-----------------------------------------------------------------------------------------------------------
--	Measure DataClock (to calculate throughput)
-----------------------------------------------------------------------------------------------------------	

measure_DataClock_inst : measure_refclk
GENERIC MAP
	(
		CYC_MEASURE_CLK_IN_1_SEC	=> SAMPLES_100MHZ
	)
port MAP(
	RefClock					=> ClkData,
	Enable					=> '1',
	Measure_Clk				=> Clk100Mhz,
	reset						=> PowerOnReset,
	RefClock_Measure		=> DataClock_Measure
	);
	
-----------------------------------------------------------------------------------------------------------
--	Measure the received datarate (actual Datathroughput)
-----------------------------------------------------------------------------------------------------------	

measure_DataOut_Clock_inst : measure_refclk
GENERIC MAP
	(
		CYC_MEASURE_CLK_IN_1_SEC	=> SAMPLES_100MHZ
	)
port MAP(
	RefClock					=> DataOut_Clock,
	Enable					=> Enable_Rx,
	Measure_Clk				=> Clk100Mhz,
	reset						=> PowerOnReset,
	RefClock_Measure		=> DataOut_Clock_Measure
	);
	
	
-----------------------------------------------------------------------------------------------------------
--	Create ErrorCount, Locked and ChannelOK
-----------------------------------------------------------------------------------------------------------	

	
process(DataOut_Clock,Reset_Rx_Path)
begin
	if Reset_Rx_Path = '1' then
			ChannelOK	<= '0';
			ErrorCount	<= (OTHERS => '0');
			Locked		<= '0';
			ChannelOK_Q	<= '0';
			correctable_errorcount <= (OTHERS => '0'); 
			uncorrectable_errorcount <= (OTHERS => '0');
	elsif rising_edge(DataOut_Clock) then

					ErrorCount(15 downto 0) 	<= Errorcount_CountPattern + Errorcount_PrbsPattern(0) + Errorcount_PrbsPattern(1) + Errorcount_PrbsPattern(2) + ErrorCount_PrbsPattern(3) ;
					Locked							<= CountLocked and PrbsLocked(0) and PrbsLocked(1) and PrbsLocked(2) and PrbsLocked(3) ;

--					correctable_errorcount 		<= FEC_Correctable_Lane(0) + FEC_Correctable_Lane(1) + FEC_Correctable_Lane(2) + FEC_Correctable_Lane(3)   ;
--					uncorrectable_errorcount 	<= FEC_UnCorrectable_Lane(0) + FEC_UnCorrectable_Lane(1) + FEC_UnCorrectable_Lane(2) + FEC_UnCorrectable_Lane(3)   ;
					
					
										
					if ((Locked = '1') AND (Errorcount = X"0000000000000000")) then
						ChannelOK 	<= '1';
					else
						 ChannelOK 	<= '0';
					end if;
					
					
					ChannelOK_Q <= ChannelOK;
--					if (ChannelOK_Q /= ChannelOK) and (ChannelOK = '0') then
--						led(7)  <= '0';
--					else
--						led(7)	<= '1';
--					end if;
	end if;
end process;

-----------------------------------------------------------------------------------------------------------
--	Create status registers and led output
-----------------------------------------------------------------------------------------------------------	

Generate_rx_freqlocked_combined:
FOR i IN 0 to NUMBER_OF_LANES-1 GENERATE
	Ones(I) <=  '1'; 		
END GENERATE;


rx_freqlocked_combined <= '1' when rx_freqlocked_Q2 = Ones else '0';
		
-- Note that the Channel_Reg is combining signals from different clock domains, this will be synchronized in the QSYS system.


Channel_Reg(15) 	<= Locked; 
Channel_Reg(14) 	<= pll_locked_Q2;
Channel_Reg(13) 	<= LaneAligned;
Channel_Reg(12) 	<= rx_freqlocked_combined;
Channel_Reg(11 downto 6) <= REFCLOCKMULTIPLIER;
Channel_Reg(5) 	<= Linkup;
Channel_Reg(4) 	<= XOFF_Received;
Channel_Reg(3) 	<= ChannelOK;
Channel_Reg(2) 	<= Error_Deskew;
Channel_Reg(1) 	<= decoder_error;
Channel_Reg(0) 	<= WordAligned ;	

process(Clk100Mhz,Reset)
begin
if Reset = '1' then
	led(7 downto 0) <= (OTHERS => '1');
elsif rising_edge(Clk100Mhz) then
		led(0)	<= not(pll_locked_Q2);	
		led(1) 	<= not(rx_freqlocked_combined);
		led(2) 	<= not(WordAligned);
		led(3) 	<= not(LaneAligned);
		led(4)   <= not(Linkup);
		led(5) 	<= not(Locked);																						
		led(6) 	<= not(ChannelOK);
		led(7)	<= not(XOFF_Received);
	end if;
end process;
	


ErrorCount_Reg <= ErrorCount;


Bitrate_Reg 	<= RefClock_Measure ; 
DataClock_Reg	<= DataClock_Measure;
DataOut_Clock_Reg	<= DataOut_Clock_Measure;

Latency_Max_Reg <= Latency_Max;
Latency_Min_Reg <= Latency_Min;



Generate_DELAY_REG:
FOR i IN 0 to NUMBER_OF_LANES-1 GENERATE
	Delay_Reg((3*I)+2 downto (3*I)) <=  Delay(I); 		
END GENERATE;

Delay_Reg_2 <= (OTHERS => '0');

-----------------------------------------------------------------------------------------------------------
--	Instantiate Temperature sensing for Arria10
-----------------------------------------------------------------------------------------------------------	


alt_a10_temp_sense_inst : alt_a10_temp_sense
port map
(
	clk 			=> Clk100Mhz,
	degrees_c 	=> Temperature,
	degrees_f   => open
);

			
end;



