


set_time_format -unit ns -decimal_places 3




#USB Blaster 1 uses 6 MHz clock = 166.666 ns period
#set t_period 166.666
#USB Blaster 2 uses 24 MHz clock = 41.666 ns period
set t_period 41.666
#USB Blaster 2 running at 16 MHz clock safe mode = 62.5 ns period
#set t_period 62.5

if { [string equal quartus_fit $::TimeQuestInfo(nameofexecutable)] } { 


create_clock -name {altera_reserved_tck} -period 41.666 [get_ports {altera_reserved_tck}] 
set_max_delay -to [get_ports { altera_reserved_tdo } ] 0 
set_clock_groups -asynchronous -group {altera_reserved_tck} 

}  else {

create_clock -name {altera_reserved_tck} -period $t_period  [get_ports {altera_reserved_tck}]

set_clock_groups -asynchronous -group {altera_reserved_tck}

##JTAG Signal Constraints

#Datasheet parameters from UBII IP on EPM570F100C5
#TCO/TSU/TH are measured w.r.t usb_clk inside UBII IP which is used to generate TCK signal
set tck_blaster_tco_max 14.603
set tck_blaster_tco_min 14.603
set tdi_blaster_tco_max 8.551
set tdi_blaster_tco_min 8.551
set tms_blaster_tco_max 9.468
set tms_blaster_tco_min 9.468
#In packed mode, TDO is sampled by rdata inside MAX
set tdo_blaster_tsu 5.831
set tdo_blaster_th -1.651

#In bitbang mode, TDO is sampled through MAX at FX2
set tdo_blaster_tpd_max 10.718
set tdo_blaster_tpd_min 10.718
set fx2_pb0_trace_max 0.152
set fx2_pb0_trace_min 0.152

#Cable delays are from USB Blaster II
#TCK
set tck_cable_max 11.627
set tck_cable_min 10.00
#*USER MODIFY* This depends on the trace length from JTAG 10-pin header to FPGA on board
set tck_header_trace_max 0.5
set tck_header_trace_min 0.1

#TMS
set tms_cable_max 11.627
set tms_cable_min 10.0
#*USER MODIFY* This depends on the trace length from JTAG 10-pin header to FPGA on board
set tms_header_trace_max 0.5
set tms_header_trace_min 0.1


#TDI
set tdi_cable_max 11.627
set tdi_cable_min 10.0
#*USER MODIFY* This depends on the trace length from JTAG 10-pin header to FPGA on board
set tdi_header_trace_max 0.5
set tdi_header_trace_min 0.1


#TDO
set tdo_cable_max 11.627
set tdo_cable_min 10.0
#*USER MODIFY* This depends on the trace length from JTAG 10-pin header to FPGA on board
set tdo_header_trace_max 0.5
set tdo_header_trace_min 0.1



derive_clock_uncertainty



#TMS
set tms_in_max [expr {$tms_cable_max + $tms_header_trace_max + $tms_blaster_tco_max - $tck_blaster_tco_min - $tck_cable_min - $tck_header_trace_min }]
set tms_in_min [expr {$tms_cable_min + $tms_header_trace_min + $tms_blaster_tco_min - $tck_blaster_tco_max - $tck_cable_max - $tck_header_trace_max }]
set_input_delay -add_delay -clock_fall -clock altera_reserved_tck -max $tms_in_max [get_ports {altera_reserved_tms}]
set_input_delay -add_delay -clock_fall -clock altera_reserved_tck -min $tms_in_min [get_ports {altera_reserved_tms}]

#TDI
set tdi_in_max [expr {$tdi_cable_max + $tdi_header_trace_max + $tdi_blaster_tco_max - $tck_blaster_tco_min - $tck_cable_min - $tck_header_trace_min }]
set tdi_in_min [expr {$tdi_cable_min + $tdi_header_trace_min + $tdi_blaster_tco_min - $tck_blaster_tco_max - $tck_cable_max - $tck_header_trace_max }]
set_input_delay -add_delay -clock_fall -clock altera_reserved_tck -max $tdi_in_max [get_ports {altera_reserved_tdi}]
set_input_delay -add_delay -clock_fall -clock altera_reserved_tck -min $tdi_in_min [get_ports {altera_reserved_tdi}]


#TDO Timing in Packed Mode (default mode)
#TDO timing delays must take into account the TCK delay from the Blaster to the FPGA TCK input pin
set tdo_out_max [expr {$tdo_cable_max + $tdo_header_trace_max + $tdo_blaster_tsu + $tck_blaster_tco_max + $tck_cable_max + $tck_header_trace_max }]
set tdo_out_min [expr {$tdo_cable_min + $tdo_header_trace_min - $tdo_blaster_th + $tck_blaster_tco_min + $tck_cable_min + $tck_header_trace_min }]
#TDO does not latch inside the USB Blaster II at the rising edge of TCK, it actually is latched one half cycle later in packed mode (equivalent to 1 JTAG fall-to-fall cycles)
set_output_delay -add_delay -clock_fall -clock altera_reserved_tck -max $tdo_out_max [get_ports {altera_reserved_tdo}]
set_output_delay -add_delay -clock_fall -clock altera_reserved_tck -min $tdo_out_min [get_ports {altera_reserved_tdo}]

#TDO Timing in Bitbang Mode (non-default mode)
#TDO timing delays must take into account the TCK delay from the Blaster to the FPGA TCK input pin
#set tdo_out_max [expr {$tdo_cable_max + $tdo_header_trace_max + $tdo_blaster_tpd_max + $fx2_pb0_trace_max + $tck_blaster_tco_max + $tck_cable_max + $tck_header_trace_max }]
#set tdo_out_min [expr {$tdo_cable_min + $tdo_header_trace_min + $tdo_blaster_tpd_min + $fx2_pb0_trace_min + $tck_blaster_tco_min + $tck_cable_min + $tck_header_trace_min }]
#set_multicycle_path -from * -to [get_ports {altera_reserved_tdo}] -setup 2 -end
#TDO does not latch inside the USB Blaster II at the rising edge of TCK, it actually is latched one half cycle later in packed mode (equivalent to 1.5 JTAG cycles)
#set_output_delay -add_delay -clock altera_reserved_tck -max $tdo_out_max [get_ports {altera_reserved_tdo}]
#set_output_delay -add_delay -clock altera_reserved_tck -min $tdo_out_min [get_ports {altera_reserved_tdo}]



# Security mode for JTAG will bypass the TAP controller and send TDI direct to TDO, unregistered.  This is not the same as JTAG BYPASS State.
# JTAG is not going to work in security mode
set_false_path -from [get_ports {altera_reserved_tdi}] -to [get_ports {altera_reserved_tdo}]

#temporary timing arc issue in 15.1
set_false_path -from altera_internal_jtag~jtag_reg -to [get_ports {altera_reserved_tdo}]

}
