

LIBRARY ieee;                                               
USE ieee.std_logic_1164.all; 
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;   
use IEEE.numeric_std.ALL;                            

ENTITY SuperliteII_Demo_tb IS
END SuperliteII_Demo_tb;

ARCHITECTURE SuperliteII_Demo_arch OF SuperliteII_Demo_tb IS
-- constants                                                 
-- signals                                                   

SIGNAL Control_Reg : STD_LOGIC_VECTOR(15 downto 0) := X"8000";
SIGNAL Control2_Reg : STD_LOGIC_VECTOR(15 downto 0) := X"8000";

CONSTANT BITRATE	: integer := 10312 ;
CONSTANT BURSTLENGTH1	: integer := 11;
CONSTANT BURSTSTART1     : integer := BITRATE * 50;
CONSTANT BURSTSTOP1      : integer := BURSTSTART1 + BURSTLENGTH1;

CONSTANT BURSTLENGTH2	: integer := 12;
CONSTANT BURSTSTART2     : integer := (BITRATE * 51) + BITRATE/2;
CONSTANT BURSTSTOP2      : integer := BURSTSTART2 + BURSTLENGTH2;


SIGNAL RefClock : STD_LOGIC := '0';
SIGNAL Clk100Mhz : STD_LOGIC := '0';
SIGNAL Enable_Core : STD_LOGIC := '1';
SIGNAL XCVR_RX : STD_LOGIC_VECTOR(3 downto 0):= (OTHERS => '0');
SIGNAL XCVR_TX : STD_LOGIC_VECTOR(3 downto 0):= (OTHERS => '0');
SIGNAL DebugOut : STD_LOGIC_VECTOR(31 downto 0):= (OTHERS => '0');
SIGNAL Channel_Reg : STD_LOGIC_VECTOR(15 downto 0) := (OTHERS => '0');
SIGNAL Counter_1ms_Reg : STD_LOGIC_VECTOR(31 downto 0) := (OTHERS => '0');
SIGNAL HSClock : STD_LOGIC := '0';
SIGNAL Shift_Reg_0 : STD_LOGIC_VECTOR(63 downto 0) := (OTHERS => '0');
SIGNAL Shift_Reg_1 : STD_LOGIC_VECTOR(63 downto 0) := (OTHERS => '0');
SIGNAL Shift_Reg_2 : STD_LOGIC_VECTOR(63 downto 0) := (OTHERS => '0');
SIGNAL Shift_Reg_3 : STD_LOGIC_VECTOR(63 downto 0) := (OTHERS => '0');
signal Blank : std_logic_vector(3 downto 0) := (OTHERS => '1');
signal Shift_Data : std_logic := '0';

signal	reconfig_reset			: std_logic := '0';
signal	reconfig_address    			: std_logic_vector(15 downto 0)   := (others => '0');
signal	reconfig_read        		: std_logic                      := '0';  
signal	reconfig_readdata    		: std_logic_vector(31 downto 0);  
signal	reconfig_waitrequest 		: std_logic;                        
signal	reconfig_write       		: std_logic                      := '0';
signal	reconfig_writedata   		: std_logic_vector(31 downto 0)  := (others => '0'); 

signal	reconfig_pll_reset			: std_logic := '0';
signal	reconfig_pll_address    			: std_logic_vector(15 downto 0)   := (others => '0');
signal	reconfig_pll_read        		: std_logic                      := '0';  
signal	reconfig_pll_readdata    		: std_logic_vector(31 downto 0);  
signal	reconfig_pll_waitrequest 		: std_logic;                        
signal	reconfig_pll_write       		: std_logic                      := '0';
signal	reconfig_pll_writedata   		: std_logic_vector(31 downto 0)  := (others => '0'); 


signal  count_hispeed : std_logic_vector(19 downto 0)  := (others => '0');
signal  Biterror : std_logic := '0';
signal XCVR_TX_new  : std_logic := '0';
signal Serial_Clock : std_logic := '0';
signal Burst1Found : std_logic := '0';
signal Burst2Found : std_logic := '0';




Signal SelectData : std_logic;

COMPONENT SuperliteII_Demo
PORT(
	RefClock						: in std_logic; 						
	Clk100Mhz					: in std_logic;
	Enable_Core					: in std_logic;
	XCVR_TX						: OUT STD_LOGIC_VECTOR (3 DOWNTO 0);	-- 10 Channels
	XCVR_RX						: IN STD_LOGIC_VECTOR (3 DOWNTO 0);     
	led							: out std_logic_vector(7 downto 0);
	Control_Reg					: in std_logic_vector(15 downto 0);
	Control2_Reg				: in std_logic_vector(15 downto 0);
	reconfig_reset          : in  std_logic; 
	reconfig_write          : in  std_logic; 
	reconfig_read           : in  std_logic; 
	reconfig_address        : in  std_logic_vector(15 downto 0)  := (others => '0'); 
	reconfig_writedata      : in  std_logic_vector(31 downto 0)  := (others => '0'); 
	reconfig_readdata       : out std_logic_vector(31 downto 0);   
	reconfig_waitrequest    : out  std_logic;
	reconfig_pll_reset          : in  std_logic; 
	reconfig_pll_write          : in  std_logic; 
	reconfig_pll_read           : in  std_logic; 
	reconfig_pll_address        : in  std_logic_vector(15 downto 0)  := (others => '0'); 
	reconfig_pll_writedata      : in  std_logic_vector(31 downto 0)  := (others => '0'); 
	reconfig_pll_readdata       : out std_logic_vector(31 downto 0);   
	reconfig_pll_waitrequest    : out  std_logic;
	Channel_Reg					: out std_logic_vector(15 DOWNTO 0);
	Counter_1ms_Reg			: out std_logic_vector(31 DOWNTO 0);
	ErrorCount_Reg				: out std_logic_vector(15 DOWNTO 0);
	Bitrate_Reg					: out std_logic_vector(31 DOWNTO 0);
	DataClock_Reg				: out std_logic_vector(31 DOWNTO 0);
	DataOut_Clock_Reg			: out std_logic_vector(31 DOWNTO 0);
	Latency_Max_Reg			: out std_logic_vector(7 downto 0);
	Latency_Min_Reg			: out std_logic_vector(7 downto 0);
	Delay_Reg					: out std_logic_vector(31 downto 0) := (OTHERS => '0');
	Delay_Reg_2					: out std_logic_vector(31 downto 0) := (OTHERS => '0');
	Temperature					: out std_logic_vector(7 downto 0)		
	);
END COMPONENT;


BEGIN




	-- Reset
	Control_Reg(15) <= '1','0' after 1 us;

	-- Insert Error at 45.5 and 47.0 us
	Control_Reg(14) <= '0','1' after 45.5 us,'0' after 45.6 us, '1' after 47.0 us,'0' after 47.1 us;

	-- Reset ErrorCounter at 45  us
	Control_Reg(13) <= '0','1' after 55 us,'0' after 55.1 us;

	-- Force Align
	Control_Reg(12) <= '0'; 
	
	-- Manual Training
	--Control_Reg(11) <= '1','0' after 50 us;
	
	-- Automatic Training (Requires loopback connection)
	Control_Reg(11) <= '0';
	Control_Reg(10 downto 0) <= (OTHERS => '0'); 
	
	Control2_Reg(15 downto 0) <= (OTHERS => '0'); -- Serial Loop disabled


	Enable_Core <= '1';




	RefClock <= not Refclock after 775.757575 ps;	-- 644.53125 Mhz 
	Clk100Mhz <= not Clk100Mhz after 5 ns;	-- 100 Mhz


	DUT : SuperliteII_Demo	PORT MAP (
-- list connections between master ports and signals
	Control_Reg => Control_Reg,
	Control2_Reg => Control2_Reg,
	Enable_Core => Enable_Core,
	RefClock => RefClock,
	Clk100Mhz	=> Clk100Mhz,
--	Disable_Rx	 => '0',
--	Disable_Tx => '0',
	XCVR_RX => XCVR_RX,
	XCVR_TX => XCVR_TX,
	Channel_Reg => Channel_Reg,
	Counter_1ms_Reg => Counter_1ms_Reg,
		reconfig_reset		      => reconfig_reset,
		reconfig_write          => reconfig_write,
		reconfig_read           => reconfig_read,
		reconfig_address        => reconfig_address,
		reconfig_writedata      => reconfig_writedata,
		reconfig_readdata       => reconfig_readdata,
		reconfig_waitrequest    => reconfig_waitrequest,
	
	-- LC PLL reconfiguration interface	
	
		reconfig_pll_reset       	=> reconfig_pll_reset,
		reconfig_pll_write       	=> reconfig_pll_write,
		reconfig_pll_read        	=> reconfig_pll_read,
		reconfig_pll_address        => reconfig_pll_address,
		reconfig_pll_writedata      => reconfig_pll_writedata,
		reconfig_pll_readdata       => reconfig_pll_readdata,
		reconfig_pll_waitrequest 	=> reconfig_pll_waitrequest	
);

--	Serial_Clock <= not Serial_Clock after 50 ps; -- 10 Ghz
	
	
--	process(Serial_Clock)
--	begin
--		if rising_edge(Serial_Clock) then
--			count_hispeed <= count_hispeed + 1;
--			if count_hispeed = X"61A80" then
--				Biterror <= '1';
--			else
--				Biterror <= '0';
--			end if;
--		end if;
--	end process;

--XCVR_TX_new <= XCVR_TX(0) XOR Biterror;


-- Insert 2 burst of errors, one to test the correctable error count, one large enough to trigger the uncorrectable error count



--	process(Serial_Clock)
--	begin
--		if rising_edge(Serial_Clock) then
--			count_hispeed <= count_hispeed + 1;
--			if (count_hispeed >= BURSTSTART1 and count_hispeed < BURSTSTOP1) or (count_hispeed >= BURSTSTART2 and count_hispeed < BURSTSTOP2) then 
--				XCVR_TX_new <= XCVR_TX(0) XOR '1'; -- Insert burst error
--				Biterror <= '1';
--				if ((Biterror = '1') and ((count_hispeed >= BURSTSTART1) and (count_hispeed < BURSTSTOP1))) then 
--					Burst1Found <= '1';
--					assert (Burst1Found = '1')
--					report "==========================================================================================================> Physical Burst of errors introduced Burst length = " & integer'image(BURSTLENGTH1)  severity note;				
--
--				end if;
--				
--				if ((Biterror = '1') and ((count_hispeed >= BURSTSTART2 and count_hispeed < BURSTSTOP2))) then 
--					Burst2Found <= '1';
--					assert (Burst2Found = '1')
--					report "==========================================================================================================> Physical Burst of errors introduced Burst length = " & integer'image(BURSTLENGTH2)  severity note;										
--				end if;				
--
--			else
--				XCVR_TX_new  <= XCVR_TX(0);
--				Biterror <= '0';
--			end if;
--		end if;
--	end process;
	


--XCVR_RX <= transport XCVR_TX;


--XCVR_RX(0) <= transport XCVR_TX_new after 0 ns;
XCVR_RX(0) <= transport XCVR_TX(0) after 0 ns; 
XCVR_RX(1) <= transport XCVR_TX(1) after 6.4 ns; -- 6.4 = 66/BITRATE  * 1000
XCVR_RX(2) <= transport XCVR_TX(2) after 0 ns;
XCVR_RX(3) <= transport XCVR_TX(3) after 6.4 ns;





                                      
END SuperliteII_Demo_arch;
