#include <stdio.h>
#include "system.h"
#include "string.h"
#include "altera_avalon_pio_regs.h"
#include <unistd.h>
#include "io.h"
#include "altera_avalon_jtag_uart_regs.h"

///////////////////////////////////////////////////////////////////////				
// Global parameters.h
///////////////////////////////////////////////////////////////////////	

#include "parameters.h"

///////////////////////////////////////////////////////////////////////				
// Local parameters
///////////////////////////////////////////////////////////////////////	

#define NUMBER_OF_LINKS 1
#define READ_LENGTH 255 // This should match the RTL implementation
#define IDLE_LENGTH 7	  // This should match the RTL implementation
#define DEBUG 0


///////////////////////////////////////////////////////////////////////				
// input_functions.c
///////////////////////////////////////////////////////////////////////	

char input_char(void);

int input_number(void);

int input_byte(void);

int input_double(void);

int input_word(void);


///////////////////////////////////////////////////////////////////////				
// PMA_functions.c
///////////////////////////////////////////////////////////////////////	

void rmw_channel (int offset,int bitmask, int newval); 

int rd_channel (int offset);

void rmw_pll (int offset,int bitmask, int newval);

int rd_pll (int offset);

int encode_pretap2 (int pretap2); 

int decode_pretap2 (int pretap2_encoded); 

int encode_pretap1 (int pretap1); 

int decode_pretap1 (int pretap1_encoded);

int encode_posttap1 (int posttap1); 

int decode_posttap1 (int posttap1_encoded); 

int encode_posttap2 (int posttap2); 

int decode_posttap2 (int posttap2_encoded);

int encode_dcgain (int dcgain);

int decode_dcgain (int dcgain_encoded);

void show_pma_settings (int dfe_enable[NUMBER_OF_LANES]);


///////////////////////////////////////////////////////////////////////				
// channel_functions.c
///////////////////////////////////////////////////////////////////////	

int Read_Channel_Reg( int SelectedChannel);

int Read_ErrorCount_L_Reg( int SelectedChannel);

int Read_ErrorCount_H_Reg( int SelectedChannel);

int Read_FEC_corr_errorcount_Reg( int SelectedChannel);

int Read_FEC_uncorr_errocount_Reg( int SelectedChannel);
 

///////////////////////////////////////////////////////////////////////				
// ODI_functions.c
///////////////////////////////////////////////////////////////////////	

	
int enc_horizontal_phase (int phase);

float get_odi_errorcount (int SelectedChannel,int EyeQInterval);

float get_odi_bitcount (int SelectedChannel);

void print_2D_eye (float BER_Array[][DimB], float ErrorCount_Array[][DimB],int *veye_top, int *veye_bottom, int *table_phase_start);

void print_1D_eye (float BER_Array[][DimB], int *table_phase_start);

float do_eye_measurement(int SelectedChannel, int EyeQInterval,int Bandwidth, int VCCER_Level, int DFE_Mode 
	, float BER_Array[][DimB], float ErrorCount_Array[][DimB],int *veye, int *veye_top,int *veye_bottom, int *table_phase_start
	, int *eye_phase, int *optimum_phase, float *Totalbits_ODI, int Verbose);


///////////////////////////////////////////////////////////////////////				
// DFE_functions.c
///////////////////////////////////////////////////////////////////////	
 
void dfe_continous_enable(int channel,int number_of_taps,int freeze_mode);

void dfe_disable(int channel);


///////////////////////////////////////////////////////////////////////				
// calibration_functions.c
///////////////////////////////////////////////////////////////////////	

void recalibrate_channel(int channel, int rate_switch);

void recalibrate_rx(int channel, int rate_switch);

void recalibrate_tx(int channel);

void recalibrate_pll(int pll_type);




void set_default_setting(void)
{
	int j;
	
  		for (j = 0; j < NUMBER_OF_LANES; j++)
		{	
		
		//Set upper 4 bits of register 0xA back to 0x0 (this seems optional)
		//rmw_channel( ((j<<10) + 0xA),0xF0,0x0);

		//Set upper 4 bits of register 0xB back to 0x0 (this seems optional)
		//rmw_channel( ((j<<10) + 0xB),0xF0,0x0);

		//Set register 0x13F bits [3:0] back to 0x3 this sets the serialization factor back to 40.
		//rmw_channel( ((j<<10) + 0x13F),0x0F,0x7);
			
		//Set register 0x110 bits [2:0] back to 0x6 this sets the serialization factor back to 40.
		//rmw_channel( ((j<<10) + 0x110),0x07,0x6);
								
	   //Set value of 0x111 bits [4:0] to 0x1
		//rmw_channel( ((j<<10) + 0x111),0x1F,0x1);
			
		


		}
	}
	
/* Functions */

int main()
{ 
	char rx_char;
  int Control_Reg;
  int Control2_Reg;
  int RefClock_Reg;
  int DataClock_Reg;
  int DataClock_Out_Reg;	

  int Bitrate;
  int Linerate;
  float Bitrate_temp;
  float Linerate_temp;
  int Userdatarate;
  float Userdatarate_temp;
  int Refclockmultiplier;
  int Channel_Reg;
  int Counter_1ms_Reg;
  float BER;
  float Totalbits;
  float Efficiency;
  int Hours;
  int Minutes;
  int Seconds;
  int temp; 
  int SelectedChannel;


  int Locked[NUMBER_OF_LINKS];
  int ChannelOK[NUMBER_OF_LINKS];
  int WordAligned[NUMBER_OF_LINKS];
  int LaneAligned[NUMBER_OF_LINKS];    
  int Powerdown[NUMBER_OF_LINKS];
  int PLL_Locked[NUMBER_OF_LINKS];
  int Rx_FreqLocked[NUMBER_OF_LINKS];
  float ErrorCount; 
  int ErrorCount_Reg;

  int tx_vodctrl[NUMBER_OF_LANES];
  int tx_pretap_1[NUMBER_OF_LANES];
  int tx_pretap_2[NUMBER_OF_LANES];
  int tx_posttap_1[NUMBER_OF_LANES];
  int tx_posttap_2[NUMBER_OF_LANES];  
  int rx_eqdcgain[NUMBER_OF_LANES];
  int rx_eqctrl[NUMBER_OF_LANES]; 
  int vga_gain[NUMBER_OF_LANES];

  int i;
  int j;
  int BERInterval;
  int TimeInterval;

  
 // int Show_ErrorCount;  
  int Reg_data;
  int Ravail;
//  int TxRatematcherFull[NUMBER_OF_LANES];
  int Serial_Loop[NUMBER_OF_LANES];
  int NOK;
  //int Loopback_All;
  


  
//ODI Variables start
  float ErrorCount_Array[DimA][DimB];
  float BER_Array[DimA][DimB];
  int table_phase_start;
  int veye_top;
  int veye_bottom;
  float eye; 
  int eye_phase;
  int optimum_phase; 
  int veye;
  float Totalbits_ODI;
  int EyeQInterval;
  int VCCER_Level;
  int Bandwidth;
  int DFE_Mode;	
//ODI Variables stop
  

//dfe variables start
  int dfe_enable[NUMBER_OF_LANES]; 
//  int dfe_taps[NUMBER_OF_LANES];
//  int freeze_mode[NUMBER_OF_LANES];
//dfe Variables stop

//    int High_nibble;
//  int Low_nibble;
  
  int Temperature;
  int Error_Deskew;
  int Error_Decoder;
  int Latency_Max;
  int Latency_Min;
  int Latency_Max_Reg;
  int Latency_Min_Reg;  
  float latency_max_measure_temp;
  float latency_min_measure_temp;  
//  int Delay_Delta_Maximum;
//  int Delay_Delta_Minimum;
//  int Delay_Maximum;
//  int Delay_Minimum;
//  int Delay_difference;
//  int Delay_memory[6];
//  int LockTime_Maximum;
//  int LockTime_Minimum; 
//  int LockTime_Average;
//  int LockTime_Total;  
  
//  int Count_Reset;
//  int Count_Reset_Maximum;
//  int Count_Reset_Minimum; 
//  int Count_Reset_Average;
//  int Count_Reset_Total;    
//  int Count_Reset_memory[6];
  int dump_register_before[600];
  int dump_register_after[600];
  
  int Latency_Memory[RESET_CYCLES];
  int Latency_Minimum;
  int Latency_Maximum;
  int hist[256];
  
//  int FEC_Correctable_ErrorCount_Reg;
//  int FEC_UnCorrectable_ErrorCount_Reg;
  
  float Ratio;
  int LinkUp[NUMBER_OF_LINKS];
  int XOFF_Received[NUMBER_OF_LINKS];
  int XOFF; 

  

   /**********************************************************************/
  /*  Overwrite fix and dyn values of CDR                                      */
  /**********************************************************************/ 

//		for (j = 0; j < NUMBER_OF_LANES; j++)
//		{	
//		
//		//Set dyn to 31 : 0x1F on the position of 0x7C becomes 0x7C;
//		rmw_channel( ((j<<10) + 0x137),0x7C,0x7C);
//		
//		//Set fix to 3 (0x3).
//		rmw_channel( ((j<<10) + 0x136),0x0F,0x03);
//		rmw_channel( ((j<<10) + 0x134),0x40,0x00);
//		
//		//set IPD to 3 (0x18)
//		rmw_channel( ((j<<10) + 0x139),0x38,0x18);	
//		}


  /**********************************************************************/
  /*  Apply temp workaround to avoid Tx flatlining	                     */
  /**********************************************************************/ 

//		for (j = 0; j < NUMBER_OF_LANES; j++)
//		{	
//		
//		rmw_channel( ((j<<10) + 0x106),0x60,0x60);
//		
//		rmw_channel( ((j<<10) + 0x107),0x60,0x60);			
//		rmw_channel( ((j<<10) + 0x115),0x57,0x57);									
//		}
		
  /**********************************************************************/
  /*  Set  cgm_bias_disable and chgpmp_replicate                        */
  /**********************************************************************/ 

		//for (j = 0; j < NUMBER_OF_LANES; j++)
		//{	
		// set cgm_bias_disable 0x11B bit [5] to 1 to improve Rx sensitivity
		//rmw_channel( ((j<<10) + 0x11B),0x20,0x20);
		// ES1 set chgpmp_replicate 0x139 bit [6] to  0
		//rmw_channel( ((j<<10) + 0x139),0x40,0x00);			
		// ES2 set chgpmp_replicate 0x139 bit [6] to  1
		//rmw_channel( ((j<<10) + 0x139),0x40,0x40);										
		//}
	

  for (j = 0; j < NUMBER_OF_LANES ; j++)
  { 
	 tx_vodctrl[j]    = VOD_31; 
	 tx_pretap_2[j]   = PRE_TAP2_0; 
    tx_pretap_1[j]   = PRE_TAP1_0;
    tx_posttap_1[j]  = POST_TAP1_0; 
    tx_posttap_2[j]  = POST_TAP2_0;
	 rx_eqdcgain[j]   = EQ_GAIN_0;
    rx_eqctrl[j]     = EQ_CTRL_8;
	 vga_gain[j]      = RADP_VGA_SEL_4; 
	 

	 //Set VOD 0x109 bit [4:0]
	 rmw_channel( ((j<<10) + 0x109),0x1F,tx_vodctrl[j]);
	  
	 //Set PRETAP2 0x108 bit [4] and [2:0]
	 rmw_channel( ((j<<10) + 0x108),0x17,tx_pretap_2[j]);	 
	  
	 //Set PRETAP1 0x107 bit [5:0]
	 rmw_channel( ((j<<10) + 0x107),0x3F,tx_pretap_1[j]);	 	
	  
	 //Set POSTTAP1 0x105 bit [6:0]
	 rmw_channel( ((j<<10) + 0x105),0x7F,tx_posttap_1[j]);	
	  
	 //Set POSTTAP2 0x106 bit [5] and [3:0]
	 rmw_channel( ((j<<10) + 0x106),0x2F,tx_posttap_2[j]);	 
	 
	 //Set AC Gain 0x167 bit [5:1]
	 rmw_channel( ((j<<10) + 0x167),0x3E, rx_eqctrl[j] << 1);
	  
	 //Set DC Gain 0x11C bit [3:0] 0x11A bit [7:0]
	 rmw_channel( ((j<<10) + 0x11C),0x0F,rx_eqdcgain[j] >> 8);		
    rmw_channel( ((j<<10) + 0x11A),0xFF,rx_eqdcgain[j]);	

	 //Set VGA Gain 0x160 bit [3:1]
	 rmw_channel( ((j<<10) + 0x160),0xE, vga_gain[j] << 1);
	 
	 Serial_Loop[j] = 0;

	 dfe_enable[j] = 0;
	
	 //dfe_taps[j]		= CONTINUOUS_11TAPS; // Use 11 taps
	 
	 //freeze_mode[j]   = 1; //Default is with freeze_mode enabled
	 

  } 
   
  
  
  
  SelectedChannel = 0; // Default Selected Channels is channel 0 

  Control_Reg = 0x0000;  
  IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);  
  Control2_Reg = 0x0000;	  //Disable serial loopback control
  IOWR_ALTERA_AVALON_PIO_DATA(CONTROL2_REG_BASE,Control2_Reg);
  
  XOFF = 0;
 

  /**********************************************************************/
  /*  Set Default settings                                                 */
  /**********************************************************************/  
  

  //set_default_setting();
  
  
 /**********************************************************************/
  /*  Reset Design                                                   */
  /**********************************************************************/  
  

        Control_Reg = Control_Reg | (0x8000);
   
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        usleep(10);
           
        Control_Reg = Control_Reg & (0x7FFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        
        do
        {
          Counter_1ms_Reg = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE); // NUMBER_OF_MS_PER_SECOND;
        } while ( Counter_1ms_Reg < 100);
    
   
   
//   Show_ErrorCount = 0;    
   
   BERInterval = 1; // Set to 1 seconds as default; 
   EyeQInterval = 1; // Set to 1 ms as default.
   // Alternative to usleep 
   //do
   // {
   //  Counter_1ms_Reg = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE); // NUMBER_OF_MS_PER_SECOND;
   //} while ( Counter_1ms_Reg < 1000);
  
   Powerdown[0] = 0;
		  
   
  /**********************************************************************/
  /*  Main Program Loop                                                 */
  /**********************************************************************/  
  while (1)
	{
 
  		
  Channel_Reg      = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL_REG_BASE);

  
  RefClock_Reg   = IORD_ALTERA_AVALON_PIO_DATA(BITRATE_REG_BASE); 
  DataClock_Reg = IORD_ALTERA_AVALON_PIO_DATA(DATACLOCK_REG_BASE);
  DataClock_Out_Reg = IORD_ALTERA_AVALON_PIO_DATA(DATAOUT_CLOCK_REG_BASE);
  Latency_Max_Reg = IORD_ALTERA_AVALON_PIO_DATA(LATENCY_MAX_REG_BASE);   
  Latency_Min_Reg = IORD_ALTERA_AVALON_PIO_DATA(LATENCY_MIN_REG_BASE);  

  Ratio = ((float)(READ_LENGTH - IDLE_LENGTH))/(READ_LENGTH);
		
  Refclockmultiplier = (((0x003F) & (Channel_Reg >> 6)));

  Bitrate_temp =  ((float)RefClock_Reg * Refclockmultiplier * NUMBER_OF_LANES)/(1000000);
  Linerate_temp =  ((float)RefClock_Reg * Refclockmultiplier)/(1000000);
  Userdatarate_temp = ((float)DataClock_Out_Reg * 64 * NUMBER_OF_LANES )/(1000000);
		
		
  latency_max_measure_temp = ((float)Latency_Max_Reg * 1000000000)/((float)DataClock_Reg);
  latency_min_measure_temp = ((float)Latency_Min_Reg * 1000000000)/((float)DataClock_Reg);
  
  Bitrate = (Bitrate_temp); // Convert the floats to integer numbers
  Linerate = (Linerate_temp); // Convert the floats to integer numbers
  Userdatarate = (Userdatarate_temp); // Convert the floats to integer numbers  
  Latency_Max = (latency_max_measure_temp); // Convert the floats to integer numbers 
  Latency_Min = (latency_min_measure_temp); // Convert the floats to integer numbers 
  
  Efficiency  = (Userdatarate * 100) /(float) (Linerate * NUMBER_OF_LANES);  
    
  Counter_1ms_Reg     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);

  ErrorCount_Reg   =  ((0x0000FFFF) & IORD_ALTERA_AVALON_PIO_DATA(BITERRORCOUNT_REG_BASE));	// Mask the upper 16 bits as they will also be read and could contain a value
  
//  FEC_Correctable_ErrorCount_Reg = ((0x0000FFFF) & IORD_ALTERA_AVALON_PIO_DATA(FEC_CORRECTABLE_ERROR_REG_BASE));
//  FEC_UnCorrectable_ErrorCount_Reg = ((0x0000FFFF) & IORD_ALTERA_AVALON_PIO_DATA(FEC_UNCORRECTABLE_ERROR_REG_BASE));

  
  ErrorCount = (float) (ErrorCount_Reg);
  
  Temperature = IORD_ALTERA_AVALON_PIO_DATA(TEMPERATURE_REG_BASE);
  

  
  for (i = 0; i < NUMBER_OF_LINKS ; i++)
  {

  Locked[i]  = 0x0001 & (Channel_Reg >> 15);    
  PLL_Locked[i]  = 0x0001 & (Channel_Reg >> 14);
  LaneAligned[i] = 0x0001 & (Channel_Reg >>  13);  
  Rx_FreqLocked[i]   = 0x0001 & (Channel_Reg >> 12); 
  LinkUp[i]   = 0x0001 & (Channel_Reg >> 5); 	
  XOFF_Received[i] = 0x0001 & (Channel_Reg >>  4);		  
  Error_Deskew = 0x0001 & (Channel_Reg >>  2);  
  Error_Decoder = 0x0001 & (Channel_Reg >>  1);	  
  WordAligned[i] = 0x0001 & (Channel_Reg >>  0);    
  }
  
  
  Totalbits = (float) ( Counter_1ms_Reg) * (float) (Bitrate*1000);

  

  Hours =  Counter_1ms_Reg / NUMBER_OF_MS_PER_HOUR ;
  Minutes = ( Counter_1ms_Reg - (Hours * NUMBER_OF_MS_PER_HOUR))/NUMBER_OF_MS_PER_MINUTE;
  Seconds = ( Counter_1ms_Reg - (Hours * NUMBER_OF_MS_PER_HOUR) - (Minutes * NUMBER_OF_MS_PER_MINUTE)) / NUMBER_OF_MS_PER_SECOND;
  
  

      if (ErrorCount > 0)
        BER = ((ErrorCount)) / Totalbits;
      else
        //BER = 0;
        BER = ((float) 3) / Totalbits; /* confidence level (CL) of 95% = -ln(1-CL) = 3*/
      

    printf("\n\n\n");
    printf("Arria 10 GX Devkit Superlite II V3 across %2d Lanes \n",NUMBER_OF_LANES);
    printf("--------------------------------------------------------------------------------\n");
    printf("|Number of Lanes         : %2d  \n",NUMBER_OF_LANES);
    printf("|Line rate               : %4d Mbps \n",Linerate);
    printf("|Aggregrate Line Rate    : %4d Mbps \n",Bitrate);  
    printf("|Read Length             : %3d \n",READ_LENGTH);  
    printf("|Idle Length             : %1d \n",IDLE_LENGTH); 
    printf("|Ratio                   : %4.4f   \n" ,Ratio);		
    printf("|Net User Data Bandwidth : %4d Mbps \n",Userdatarate);  
    printf("|Efficiency              : %4.2f %% \n",Efficiency); 		
    printf("|Measured Max Latency    : %4d ns, %2d parallel clocks\n",Latency_Max,Latency_Max_Reg);  
    printf("|Measured Min Latency    : %4d ns, %2d parallel clocks\n",Latency_Min,Latency_Min_Reg); 		
    printf("|EyeQInterval            : %4d ms \n",EyeQInterval);   
    printf("|SelectedChannel         : "); 
      switch (SelectedChannel)
       {
       case  0  : printf(" 0 (QSFP+ Lane 0) \n"); break;
       case  1  : printf(" 1 (QSFP+ Lane 1) \n"); break;
       case  2  : printf(" 2 (QSFP+ Lane 2) \n"); break;
       case  3  : printf(" 3 (QSFP+ Lane 3) \n"); break;		 
       default : break;
       }  		
     printf("-------------------------------------------------------------------------------\n");
		

    printf("\n");

     
    printf("Status\n");
    printf("======\n");     

    printf("Lane            |");
    for (i = 0; i < NUMBER_OF_LANES ; i++)
    { 
      printf("%2X|",i);
    }
    printf("\n");
    printf("                |");
    for (i = 0; i < NUMBER_OF_LANES ; i++)
    { 
    printf("--|");
    }
    printf("\n");

    printf("Serial Loop     :");        
    for (i = 0; i < NUMBER_OF_LANES ; i++)
    {     
      printf("%2d|",Serial_Loop[i]);   
    }
    printf("\n"); 
	              
    printf("PLLLocked       :");        
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    {
    if (Powerdown[i] == 0) 
      printf("%2d",PLL_Locked[i]);
    else
      printf("  ");    
    }
    printf("\n"); 
    
    printf("FreqLocked      :");        
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    {
    if (Powerdown[i] == 0)       
      printf("%2d",Rx_FreqLocked[i]);
    else
      printf("  ");    
    }
    printf("\n");

    printf("Error Deskew    :");       
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    {
    if (Powerdown[i] == 0)       
      printf("%2d",Error_Deskew);
    else
      printf("  ");    
    }
    printf("\n"); 

    printf("XOFF Sent       :");       
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    {
    if (Powerdown[i] == 0)       
      printf("%2d",XOFF);
    else
      printf("  ");    
    }
    printf("\n"); 	 

    printf("XOFF Received   :");       
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    {
    if (Powerdown[i] == 0)       
      printf("%2d",XOFF_Received[i]);
    else
      printf("  ");    
    }
    printf("\n"); 
	 

    printf("Error Decoder   :");       
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    {
    if (Powerdown[i] == 0)       
      printf("%2d",Error_Decoder);
    else
      printf("  ");    
    }
    printf("\n"); 	 
 
    printf("WordAligned     :");       
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    {
    if (Powerdown[i] == 0)       
      printf("%2d",WordAligned[i]);
    else
      printf("  ");    
    }
    printf("\n");     

    printf("LaneAligned     :");       
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    {
    if (Powerdown[i] == 0)       
      printf("%2d",LaneAligned[i]);
    else
      printf("  ");    
    }
    printf("\n");  

    printf("LinkUp          :");       
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    {
    if (Powerdown[i] == 0)       
      printf("%2d",LinkUp[i]);
    else
      printf("  ");    
    }
    printf("\n");
	 

      
    printf("DataLocked      :");

    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    {
    if (Powerdown[i] == 0)      
      printf("%2d",Locked[i]);
    else
      printf("  ");
    }
    printf("\n");
  
      
    printf("Errorcount      :");
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    {
        if ((Locked[i] == 0) || (Powerdown[i] == 1))
          printf("  ");
        else if (ErrorCount > 99)
          printf("xx");
        else    
          printf("%2d",ErrorCount_Reg);   
    }
    printf("\n\n\n");

//	 printf("FEC Correctable Errors     :");
//    for (i = 0; i < NUMBER_OF_LINKS ; i++)
//    {
//        if ((Locked[i] == 0) || (Powerdown[i] == 1))
//          printf("  ");
//        else    
//          printf("%4d", FEC_Correctable_ErrorCount_Reg);   
//    }
//    printf("\n");	
//	
//	 printf("FEC Uncorrectable Errors   :");
//    for (i = 0; i < NUMBER_OF_LINKS ; i++)
//    {
//        if ((Locked[i] == 0) || (Powerdown[i] == 1))
//          printf("  ");
//        else    
//          printf("%4d", FEC_UnCorrectable_ErrorCount_Reg);   
//    }
//    printf("\n\n");		 
 

    
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    { 
      if (Powerdown[i] == 1)
        printf("Ch%2d Powered down\n",i);        
      else if (Locked[i] == 1)
      {
        if (ErrorCount > 0)
              printf("BER Estimate Link          : %e    (# Errors/Totalbits)\n",BER);
          else
              printf("BER (CL=0.95) Link         : %e\n",BER);
      }
      else
        printf("No Data Lock achieved \n");
    }
    
 
     printf("\n");
     
    printf("Test Time                  : %2dh %2dm %2ds\n",Hours,Minutes,Seconds);             
    printf("Reference Clock Frequency  : %7d kHz\n",(RefClock_Reg)/1000); 
    printf("DataClock Frequency        : %7d kHZ\n",DataClock_Reg/1000); 
	 printf("DataClock_out Frequency    : %7d kHZ\n",DataClock_Out_Reg/1000); 
    printf("Temperature Arria10        : %3d (degrees Celcius)\n",Temperature);	 
    printf("\n");
       
    printf("\nSelect Action : \n");
    printf("===============\n");
    
    printf("1. Set all channels back to default and toggle Reset\n");
    printf("2. Force Re-Alignment on Rx Path\n");  
    printf("3. Select Channel To Control\n");	 
    printf("4. Show/Control Transceiver PMA Settings on Links\n");
    printf("5. Insert Biterrors on Link (%1d at a time)\n",(NUMBER_OF_LANES));
    printf("6. Reset ErrorCounter\n");
    printf("7. Show Status\n");
    printf("8. Control Serial loopback\n");
    printf("C. Refresh BER every %d seconds \n",BERInterval);
    printf("D. Input new BER Time Interval\n");
    printf("E. Show Transceiver PMA Settings on all channels\n");	 
	 printf("G. Input new EyeQInterval time\n");	 
    //printf("M. Perform %4d resets and measure latency and delay\n",RESET_CYCLES);	 
	 //printf("P. Redo ATX PLL calibration \n");
	 //printf("Q. Change Clocking to non-bonded for TTK to work properly \n");
	 printf("S. Store all channel information in memory of Selected Channel \n");
	 printf("T. Store and compare with values stored in process S \n");	 
	 printf("X. Toggle XOFF to partner to stop/start sending traffic at remote side \n");	 	 
	 printf("Z. Dump channel content of selected channel \n");
	 printf("O. Perform ODI on selected channel \n");	 
 	 
    printf("0. Stop Test \n\n");
    printf("  Enter Choice :");
  
    rx_char = input_char(); 
    
    
		switch (rx_char)
		{
      case '1': /* Reset Link*/
 		  set_default_setting();
		
        Control_Reg = Control_Reg | (0x8000);
		  IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);

        usleep(10);
             
        Control_Reg = Control_Reg & (0x7FFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        
        
        usleep(1000000); // Wait for 1 second
        printf("All Channels set back to default and reset toggled.\n\n\n");
        break; //case '1'


      case '2': /*Force Re-Alignment on Rx Path */

        Control_Reg = Control_Reg | (0x1000);
        
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        usleep(10);
        
    
        Control_Reg = Control_Reg & (0xEFFF);
        
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        usleep(10);
        
        printf("Force Re-Alignment on Rx Path.\n\n\n");
        break; //case '2'

     case '3': /*Select Channel To Control*/

      printf("3. Select Channel To Control\n");
      printf("Current Channel : ");
      switch (SelectedChannel)
       {
       case  0  : printf(" 0: QSFP+ Lane 0 \n"); break;
       case  1  : printf(" 1: QSFP+ Lane 1 \n"); break;
       case  2  : printf(" 2: QSFP+ Lane 2 \n"); break;
       case  3  : printf(" 3: QSFP+ Lane 3 \n"); break;
//       case  4  : printf(" 4: FMCA_DP_P4 Bitrate %4d Mbps \n",Bitrate); break;
//       case  5  : printf(" 5: FMCA_DP_P5 Bitrate %4d Mbps \n",Bitrate); break;
//       case  6  : printf(" 6: FMCA_DP_P6 Bitrate %4d Mbps \n",Bitrate); break;
//       case  7  : printf(" 7: FMCA_DP_P7 Bitrate %4d Mbps \n",Bitrate); break;
//       case  8  : printf(" 8: FMCA_DP_P8 Bitrate %4d Mbps \n",Bitrate); break;
//       case  9  : printf(" 9: FMCA_DP_P9 Bitrate %4d Mbps \n",Bitrate); break;
//       case  10  : printf(" 10: FMCA_DP_P10 Bitrate %4d Mbps \n",Bitrate); break;
//       case  11  : printf(" 11: FMCA_DP_P11 Bitrate %4d Mbps \n",Bitrate); break;			 
       default : break;
       }  
 		 
     temp = 0xffffffff;
  
     do {
      printf("New Channel (0-3)         :");
      rx_char = input_char(); 
  
      switch (rx_char)
      {
      case '0': temp = 0; break;
      case '1': temp = 1; break;
      case '2': temp = 2; break;
      case '3': temp = 3; break;
//      case '4': temp = 4; break;
//      case '5': temp = 5; break;
//      case '6': temp = 6; break;
//      case '7': temp = 7; break;
//      case '8': temp = 8; break;
//      case '9': temp = 9; break;	
//		case 'a':
//		case 'A': temp = 10;break;
//		case 'b':
//		case 'B': temp = 11;break;			
      default : break;
      }
  
    } while (temp == 0xffffffff);
    
    SelectedChannel = temp;
          
          
                   

        
      break; //case '3'
	 

 			case '4': /*Show/Control Transceiver Parameters */
				
//Read settings first (as they can have been set differnetly in e.g. Transceiver toolkit or through scripts

	 tx_vodctrl[SelectedChannel]    = rd_channel((SelectedChannel << 10) + 0x109) & 0x1F; 
	 tx_pretap_2[SelectedChannel]   = rd_channel((SelectedChannel << 10) + 0x108) & 0x17; 
    tx_pretap_1[SelectedChannel]   = rd_channel((SelectedChannel << 10) + 0x107) & 0x3F; 
    tx_posttap_1[SelectedChannel]  = rd_channel((SelectedChannel << 10) + 0x105) & 0x7F; 
    tx_posttap_2[SelectedChannel]  = rd_channel((SelectedChannel << 10) + 0x106) & 0x2F; 	 
	 rx_eqctrl[SelectedChannel]     = (rd_channel((SelectedChannel << 10) + 0x167) & 0x3E) >> 1;
	 rx_eqdcgain[SelectedChannel]   = ((rd_channel((SelectedChannel << 10) + 0x11C) & 0x0F) << 8) + (rd_channel((SelectedChannel << 10) + 0x11A) & 0xFF);
	 vga_gain[SelectedChannel]      = (rd_channel((SelectedChannel << 10) + 0x160) & 0xE) >> 1;	   
	   
		  
	  printf("\n");
        printf("4. Transceiver PMA Settings of Channel %1d\n",SelectedChannel);
		  printf("    VOD Level             : %2d\n",tx_vodctrl[SelectedChannel]);
		  printf("    Pre Tap 2 Level       : %2d\n",decode_pretap2(tx_pretap_2[SelectedChannel]));
		  printf("    Pre Tap 1 Level       : %2d\n",decode_pretap1(tx_pretap_1[SelectedChannel]));				  
        printf("    Post Tap 1 Level      : %2d\n",decode_posttap1(tx_posttap_1[SelectedChannel]));   
		  printf("    Post Tap 2 Level      : %2d\n",decode_posttap2(tx_posttap_2[SelectedChannel])); 
		  printf("    CTLE DC Gain          : %2d\n",decode_dcgain(rx_eqdcgain[SelectedChannel])); 
		  printf("    CTLE AC Gain          : %2d\n",rx_eqctrl[SelectedChannel]);
		  printf("    VGA Gain              : %2d\n",vga_gain[SelectedChannel]);			  
			  
			  printf("\n");
			  
 
        printf("    New VOD Level Channel %2d               \n",SelectedChannel);
        printf("    ==================================              \n");
        printf("    Provide number from 12-31 : "); 
		  do 
		  {
		  temp = input_double(); 
		  } while ((temp < 12) || (temp > 31));
		  
		  printf("%2d",temp);
        tx_vodctrl[SelectedChannel] = temp; 
        
        printf("\n\n");  
       

        printf("    New Pre Tap 2 Level Channel %2d               \n",SelectedChannel);
        printf("    ==================================               \n");
        printf("    Provide number from 0-7 : "); 
		  do 
		  {
		  temp = input_number(); 
		  } while (temp > 7);		  
		  printf("%2d",temp);
        printf("\n\n"); 	
		  
        printf("    New Pre Tap 2 Polarity Channel %2d               \n",SelectedChannel);
        printf("    ==================================               \n");
        printf("    Provide either + or -   : ");  
//		  do 
//		  {
		  rx_char = input_char(); 
//		  printf("%c",rx_char);
//		  } while ((rx_char != '+') || (rx_char != '-'));		  // condition is not accepted?
		  
		  
		  if (rx_char == '-')
			  tx_pretap_2[SelectedChannel] = 0x10 + temp;
		  else
			  tx_pretap_2[SelectedChannel] = temp;       
           
        printf("\n\n");


        printf("    New Pre Tap 1 Level Channel %2d               \n",SelectedChannel);
        printf("    ==================================                \n");
        printf("    Provide number from 0-12 : "); 
  		  do 
		  {
		  temp = input_double(); 
		  } while ((temp < 0) || (temp > 12)); 
		  printf("%2d",temp);
        printf("\n\n");		  
        printf("    New Pre Tap 1 Polarity Channel %2d               \n",SelectedChannel);
        printf("    ==================================                \n");
        printf("    Provide either + or -    : ");  
		  rx_char = input_char();
		  
		  
		  if (rx_char == '-')
			  tx_pretap_1[SelectedChannel] = 0x20 + temp;
		  else
			  tx_pretap_1[SelectedChannel] = temp;       
           
        printf("\n\n");
    
        
        printf("    New Post Tap 1 Level Channel %2d               \n",SelectedChannel);
        printf("    ==================================              \n");
        printf("    Provide number from 0-25 : "); 
		  do 
		  {
		  temp = input_double(); 
		  } while ((temp < 0) || (temp > 25));
		  printf("%2d",temp);
        printf("\n\n");	
		  
        printf("    New Post Tap 1 Polarity Channel %2d               \n",SelectedChannel);
        printf("    ==================================              \n");
        printf("    Provide either + or -    : ");  
		  rx_char = input_char();
		  
		  
		  if (rx_char == '-')
			  tx_posttap_1[SelectedChannel] = 0x40 + temp;
		  else
			  tx_posttap_1[SelectedChannel] = temp;       
           
        printf("\n\n");  

 
        printf("    New Post Tap 2 Level Channel %2d               \n",SelectedChannel);
        printf("    ==================================                \n");
        printf("    Provide number from 0-12 : "); 
		  do 
		  {
		  temp = input_double(); 
		  } while ((temp < 0) || (temp > 12)); 
		  printf("%2d",temp);
        printf("\n\n");
		  
        printf("    New Post Tap 2 Polarity Channel %2d               \n",SelectedChannel);
        printf("    ==================================                \n");
        printf("    Provide either + or -    : ");  
		  rx_char = input_char();
		  
		  
		  if (rx_char == '-')
			  tx_posttap_2[SelectedChannel] = 0x20 + temp;
		  else
			  tx_posttap_2[SelectedChannel] = temp;   
		  
		   printf("\n\n"); 

        printf("    New CTLE AC Gain Channel %2d               \n",SelectedChannel);
        printf("    ==================================               \n");
        printf("    Provide number from 0-31 : "); 
		  do 
		  {
		  temp = input_double(); 
		  } while ((temp < 0) || (temp > 31));
		  printf("%2d",temp);
        printf("\n\n");
		  
		  rx_eqctrl[SelectedChannel] = temp; 
		  
		  printf("\n"); 


        printf("    New CTLE DC Gain Channel %2d               \n",SelectedChannel);
        printf("    ==================================               \n");
        printf("    Provide number from 0-4 : "); 
		  do 
		  {
		  temp = input_number(); 
		  } while (temp > 4);
		  printf("%2d",temp);
        printf("\n\n");
		  
		  rx_eqdcgain[SelectedChannel] = encode_dcgain(temp); 
		  
		  printf("\n");


        printf("    New VGA Gain Channel %2d               \n",SelectedChannel);
        printf("    ==================================               \n");
        printf("    Provide number from 0-7 : "); 
		  do 
		  {
		  temp = input_number(); 
		  } while ((temp < 0) || (temp > 7));
		  printf("%2d",temp);
        printf("\n\n");
		  
		  vga_gain[SelectedChannel] = temp; 
		  
		  
		  printf("\n"); 
        
        // Debug Purposes
         printf("    tx_vodctrl   = %d               \n",tx_vodctrl[SelectedChannel]);
         printf("    tx_pretap_2  = %x              \n",tx_pretap_1[SelectedChannel]);		  
         printf("    tx_pretap_1  = %x              \n",tx_pretap_1[SelectedChannel]);
         printf("    tx_posttap_1 = %x              \n",tx_posttap_1[SelectedChannel]);
         printf("    tx_posttap_2 = %x              \n",tx_posttap_2[SelectedChannel]);
         printf("    rx_eqdcgain  = %x              \n",rx_eqdcgain[SelectedChannel] );
         printf("    rx_eqctrl    = %d              \n",rx_eqctrl[SelectedChannel] );
		   printf("    vga_gain     = %d              \n",vga_gain[SelectedChannel] );
		  
			 //Set VOD 0x109 bit [4:0]
			 rmw_channel( ((SelectedChannel<<10) + 0x109),0x1F,tx_vodctrl[SelectedChannel]);
			  
			 //Set PRETAP2 0x108 bit [4] and [2:0]
			 rmw_channel( ((SelectedChannel<<10) + 0x108),0x17,tx_pretap_2[SelectedChannel]);	 
			  
			 //Set PRETAP1 0x107 bit [5:0]
			 rmw_channel( ((SelectedChannel<<10) + 0x107),0x3F,tx_pretap_1[SelectedChannel]);	 	
			  
			 //Set POSTTAP1 0x105 bit [6:0]
			 rmw_channel( ((SelectedChannel<<10) + 0x105),0x7F,tx_posttap_1[SelectedChannel]);	
			  
			 //Set POSTTAP2 0x106 bit [5] and [3:0]
			 rmw_channel( ((SelectedChannel<<10) + 0x106),0x2F,tx_posttap_2[SelectedChannel]);	 
			 
			 //Set AC Gain 0x167 bit [5:1]
			 rmw_channel( ((SelectedChannel<<10) + 0x167),0x3E, rx_eqctrl[SelectedChannel] << 1);
			  
			 //Set DC Gain 0x11C bit [3:0] 0x11A bit [7:0]
			 rmw_channel( ((SelectedChannel<<10) + 0x11C),0x0F,rx_eqdcgain[SelectedChannel] >> 8);		
			 rmw_channel( ((SelectedChannel<<10) + 0x11A),0xFF,rx_eqdcgain[SelectedChannel]);		  		  

			 //Set VGA Gain 0x160 bit [3:1]
			 rmw_channel( ((SelectedChannel<<10) + 0x160),0xE, vga_gain[SelectedChannel] << 1);

                      
				printf("\nTransceiver PMA parameters of Channel %1d updated\n\n",SelectedChannel);
				
				break; //case '4'
        
		case '5': /* Insert Biterrors */
     
        printf("5. Insert Biterror on Link\n\n\n");


        Control_Reg = Control_Reg | (0x4000);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        
        Control_Reg = Control_Reg & (0xBFFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);

       
         
		break; /* Case 5 */
	
		case '6': /* Reset ErrorCounter */
        
	      printf("6. Reset ErrorCounter \n\n\n");
   
        
        Control_Reg = Control_Reg | (0x2000);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        
        Control_Reg = Control_Reg & (0xDFFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
         
        
        usleep(100000);     
            
       break;
	
      case '7': /* Show Status */ 
            
       break;    

      case '8': /*Toggle Serial Loop SMA Lanes */
    	
    	printf("\nCurrent Status of Serial Loopback :\n");
    

		printf("Lane              |");
		for (i = 0; i < NUMBER_OF_LANES ; i++)
		{ 
		  printf("%2X|",i);
		}
		printf("\n");
		printf("                  |");
		for (i = 0; i < NUMBER_OF_LANES ; i++)
		{ 
		printf("--|");
		}
		printf("\n");
	
		printf("Serial Loop       :");        
		for (i = 0; i < NUMBER_OF_LANES ; i++)
		{     
		  printf("%2d|",Serial_Loop[i]);   
		}
		printf("\n"); 
  
              do {
               printf("\n");  
               
              printf("    Toggling Serial Loopback               \n");
               printf("    =========================              \n");
               printf("    Toggle Loopback on Lane  0  choose '0' \n");
//               printf("    Toggle Loopback on Lane  1  choose '1' \n");
//               printf("    Toggle Loopback on Lane  2  choose '2' \n");
//               printf("    Toggle Loopback on Lane  3  choose '3' \n");	
//               printf("    Toggle Loopback on Lane  4  choose '4' \n");
//               printf("    Toggle Loopback on Lane  5  choose '5' \n");
//               printf("    Toggle Loopback on Lane  6  choose '6' \n");
//               printf("    Toggle Loopback on Lane  7  choose '7' \n");
//               printf("    Toggle Loopback on Lane  8  choose '8' \n");
//               printf("    Toggle Loopback on Lane  9  choose '9' \n");
//               printf("    Toggle Loopback on Lane 10  choose 'A' \n");
//               printf("    Toggle Loopback on Lane 11  choose 'B' \n");					  
               printf("    Enable Loopback on all Lanes choose 'O' \n");
               printf("    Disable Loopback on all Lanes choose 'F' \n");                    
               printf("    No Change                 choose 'X' \n\n");    
               printf("    Choice :");      
               
               temp = 0xffffffff;
                                                    
               rx_char   = input_char();
               switch (rx_char)
               {
               case '0': temp =  0; 
							if (Serial_Loop[0] == 1)
								Serial_Loop[0] = 0;
							else
								Serial_Loop[0] = 1;
						 break;
//               case '1': temp =  1; 
//							if (Serial_Loop[1] == 1)
//								Serial_Loop[1] = 0;
//							else
//								Serial_Loop[1] = 1;
//						 break;
//               case '2': temp =  2; 
//							if (Serial_Loop[2] == 1)
//								Serial_Loop[2] = 0;
//							else
//								Serial_Loop[2] = 1;
//						 break;
//               case '3': temp =  3; 
//							if (Serial_Loop[3] == 1)
//								Serial_Loop[3] = 0;
//							else
//								Serial_Loop[3] = 1;
//						 break;
//               case '4': temp =  4; 
//							if (Serial_Loop[4] == 1)
//								Serial_Loop[4] = 0;
//							else
//								Serial_Loop[4] = 1;
//						 break;
//               case '5': temp =  5; 
//							if (Serial_Loop[5] == 1)
//								Serial_Loop[5] = 0;
//							else
//								Serial_Loop[5] = 1;
//						 break;
//               case '6': temp =  6; 
//							if (Serial_Loop[6] == 1)
//								Serial_Loop[6] = 0;
//							else
//								Serial_Loop[6] = 1;
//						 break;
//               case '7': temp =  7; 
//							if (Serial_Loop[7] == 1)
//								Serial_Loop[7] = 0;
//							else
//								Serial_Loop[7] = 1;
//						 break;
//               case '8': temp =  8; 
//							if (Serial_Loop[8] == 1)
//								Serial_Loop[8] = 0;
//							else
//								Serial_Loop[8] = 1;
//						 break;
//               case '9': temp =  9; 
//							if (Serial_Loop[9] == 1)
//								Serial_Loop[9] = 0;
//							else
//								Serial_Loop[9] = 1;
//						 break;
//               case 'a': 
//					case 'A': temp =  10; 
//							if (Serial_Loop[10] == 1)
//								Serial_Loop[10] = 0;
//							else
//								Serial_Loop[10] = 1;
//						 break;
//               case 'b': 
//					case 'B': temp =  11; 
//							if (Serial_Loop[11] == 1)
//								Serial_Loop[11] = 0;
//							else
//								Serial_Loop[11] = 1;
//						 break;							
               case 'o':
               case 'O': temp =  12; 
						 for (i = 0; i < NUMBER_OF_LANES ; i++)
							{ 
							Serial_Loop[i] = 1;
							}
						 break;

               case 'f':
               case 'F': temp =  16; 
						 for (i = 0; i < NUMBER_OF_LANES ; i++)
							{ 
							Serial_Loop[i] = 0;								
							}
						 break;
																	                        
               case 'x': 
               case 'X': temp =  128 ;break;
               default : break;
               }
               } while (temp == 0xffffffff);        	
   

 
          Control2_Reg = (Control2_Reg & (0xFFFE)) | (Serial_Loop[0]);
//          Control2_Reg = (Control2_Reg & (0xFFFD)) | (Serial_Loop[1] << 1);
//          Control2_Reg = (Control2_Reg & (0xFFFB)) | (Serial_Loop[2] << 2);
//          Control2_Reg = (Control2_Reg & (0xFFF7)) | (Serial_Loop[3] << 3); 
//			 Control2_Reg = (Control2_Reg & (0xFFEF)) | (Serial_Loop[4] << 4);
//			 Control2_Reg = (Control2_Reg & (0xFFDF)) | (Serial_Loop[5] << 5);   
//			 Control2_Reg = (Control2_Reg & (0xFFBF)) | (Serial_Loop[6] << 6);
//			 Control2_Reg = (Control2_Reg & (0xFF7F)) | (Serial_Loop[7] << 7); 
//			 Control2_Reg = (Control2_Reg & (0xFEFF)) | (Serial_Loop[8] << 8);
//			 Control2_Reg = (Control2_Reg & (0xFDFF)) | (Serial_Loop[9] << 9);	
//	       Control2_Reg = (Control2_Reg & (0xFBFF)) | (Serial_Loop[10] << 10);
//          Control2_Reg = (Control2_Reg & (0xF7FF)) | (Serial_Loop[11] << 11);  
					
		  IOWR_ALTERA_AVALON_PIO_DATA(CONTROL2_REG_BASE,Control2_Reg);
					
        

        Control_Reg = Control_Reg | (0x8000);

        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        usleep(10);
              
   
        Control_Reg = Control_Reg & (0x7FFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        
        do
        {
          Counter_1ms_Reg = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE); // NUMBER_OF_MS_PER_SECOND;
        } while ( Counter_1ms_Reg < 1000);
        
        printf("Changed serial loop and Link Resetted.\n\n\n");
        break; //case '8'     

     

      
 
     
     case 'c': //continuously update BER and errorcount every BERInterval seconds
     case 'C': 

        printf("\n");  
        printf("Show BER + ErrorCount every %d seconds              \n",BERInterval);
        printf("Press any key + Enter to stop the loop\n");
        printf("=========================================               \n");
        
        temp = 0;
        
        Reg_data = IORD_ALTERA_AVALON_JTAG_UART_DATA(ALT_STDIN_BASE);
        // Shift rigth of 16 to get access to the number of characters remaining in the read fifo
        Ravail = Reg_data >> 16;
        
        //printf("\nRavail %d :\n",Ravail);
        
        do
          {
        
        // Reset ErrorCount on Selected Channel

            Control_Reg = Control_Reg | (0x2000);
            IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
            
            Control_Reg = Control_Reg & (0xDFFF);
            IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);

            
        TimeInterval = BERInterval * 1000;
        

                do
                {
                Counter_1ms_Reg     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);
                } while ( Counter_1ms_Reg < TimeInterval);                   
             
       
        
            //Read ErrorCount registers
            
              Counter_1ms_Reg     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);
 
					ErrorCount_Reg  		 =  ((0x0000FFFF) & IORD_ALTERA_AVALON_PIO_DATA(BITERRORCOUNT_REG_BASE));


                             
   
           for (i = 0; i < NUMBER_OF_LINKS ; i++)
              {
                ErrorCount = (float) (ErrorCount_Reg);
              }

              Totalbits = (float) ( Counter_1ms_Reg) * (float) (Bitrate*1000);
  
            for (i = 0; i < NUMBER_OF_LINKS ; i++)
            {
                if (ErrorCount > 0)
                  BER = ((ErrorCount)) / Totalbits;
                else
                  BER = 0; 
            } /* For Loop */   
            

      
            if (Locked[0] == 1)
                {
                if (ErrorCount > 0)
                        printf("Link : BER Estimate : %e    ErrorCount  : %e \n",BER,ErrorCount);
                    else
                        printf("Link : BER          : 0\n");
                }
            else
              printf("No Data Lock on Link \n");
         
                
           
        temp = temp + 1;
                                           
 
        //} while (temp <= 1000); // loop a 1000 times
        Reg_data = IORD_ALTERA_AVALON_JTAG_UART_DATA(ALT_STDIN_BASE);
        Ravail = Reg_data >> 16;
        } while(!Ravail);
        rx_char   = input_char();
        
        printf("\n Stopped\n");
        
        break; // Case '9'
		  
		  

      case 'd':
      case 'D': /* New BER Time Interval */
        
        printf("D. Provide BER Time Interval\n");
        printf("   Current BER Time Interval   : %2d seconds\n", BERInterval);
        printf("   Insert new BER Interval, Hexadecimal Number from 01 to FF : ");    
        
        BERInterval = input_byte();
              
        printf("\n    New Time interval is %2d seconds:\n",BERInterval);               
            
       break; /* Case D */

      case 'g':
      case 'G': /* New EyeQ Interval Time Interval */
        
        printf("G. Provide EyeQ Time Interval\n");
        printf("   Current EyeQTime Interval   : %4d miliseconds\n", EyeQInterval);
        printf("   Insert new EyeQ Interval Time, Hexadecimal Number from 01 to FF :");		
  
		  EyeQInterval = input_byte();
            
        printf("\n    New EyeQTime interval is %4d miliseconds:\n",EyeQInterval);               
            
       break; /* Case G */		

      case 'e': /* Show Transceiver PMA Settings on all channels */
      case 'E':
			
			   show_pma_settings (dfe_enable);
			
									
    	  break; // Case E
		  

	 case 'l':
     case 'L': /* Loop reset */
       NOK = 0;
       for (j = 0; j < 1000; j++)
       {

        Control_Reg = Control_Reg | (0x8000);
        
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        usleep(10);
        
        
     
        Control_Reg = Control_Reg & (0x7FFF);
        
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        
        do
        {
          Counter_1ms_Reg = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE); // NUMBER_OF_MS_PER_SECOND;
        } while ( Counter_1ms_Reg < 100);
        
        Channel_Reg      = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL_REG_BASE);
        ChannelOK[0]  = 0x0001 & (Channel_Reg >> 3);     
		
		if (ChannelOK[0] == 0) 
			{
			NOK = NOK + 1;
			printf("\nLink NOK  press any key: %d :\n",NOK);
			rx_char = input_char(); 
			break;

			}
		else
			printf(" %d",j);
			
        } 
        printf("\n\nTotal NOK : %d", NOK);
        break; //case 'l'


	
	 case 'm':
     case 'M': /* Loop to measure latency and delay */
       NOK = 0;
	   Latency_Minimum = 1000;
		Latency_Maximum = 0;
	   //Delay_Delta_Minimum = 10;	  
	  	//Delay_Delta_Maximum = 0;	  
      
	   for (j = 0; j < 256; j++)
       {
				hist[j] = 0;
		 }
	  
       for (j = 0; j < RESET_CYCLES; j++)
       {

        Control_Reg = Control_Reg | (0x8000);

        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        usleep(100);
        
        
     
        Control_Reg = Control_Reg & (0x7FFF);

        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
		  
 		  usleep(1000); 
			 
		  do
        {
			Channel_Reg      = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL_REG_BASE);
			ChannelOK[0]  = 0x0001 & (Channel_Reg >> 3);     
        } while (ChannelOK[0] == 0);
		  // Problem with the loop above is that it will hang if the channel is not ok.
		  
 		  usleep(1000);      
		 
//		  do
//        {
//          Counter_1ms_Reg = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE); // NUMBER_OF_MS_PER_SECOND;
//        } while ( Counter_1ms_Reg < 100);
        
//        Channel_Reg      = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL_REG_BASE);
//        ChannelOK[0]  = 0x0001 & (Channel_Reg >> 3); 
		  
		  Latency_Max_Reg = IORD_ALTERA_AVALON_PIO_DATA(LATENCY_MAX_REG_BASE);   
//		  if (Latency_Max_Reg == 0)
//		  {
//				printf("\nLatency_Max_Reg is 0");
//				rx_char = input_char();
//		  }
		  
//		  if (Latency_Max_Reg > 60)
//		  {
//				printf("\nLatency_Max_Reg is bigger than 60 : %d",Latency_Max_Reg);
//				rx_char = input_char();
//		  }
		  
		  Latency_Min_Reg = IORD_ALTERA_AVALON_PIO_DATA(LATENCY_MIN_REG_BASE);  
        
		  // Measure time how long it took to reach ChannelOK
        Counter_1ms_Reg = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE); // NUMBER_OF_MS_PER_SECOND;

		if ((Latency_Max_Reg) >= 54 && (Latency_Max_Reg <= 65))
			Latency_Memory[i] = Latency_Max_Reg;
		else
			Latency_Memory[i] = 0;
		
		  
		if (ChannelOK[0] == 0) 
			{
			NOK = NOK + 1;
 
			if (Latency_Memory[i]  < Latency_Minimum )
				Latency_Minimum = Latency_Memory[i] ;
			if (Latency_Memory[i]  > Latency_Maximum )
				Latency_Maximum = Latency_Memory[i] ;
						
		  
						//printf("\n");
					   //printf("%4d:Latency:%3d clock cycles ",j,Latency_Memory[i]);	
			if (STOP == 1)
			{
				printf("\nLink NOK  press any key: %d :\n",NOK);
				rx_char = input_char();
			}
			}
		else
		{
			if (Latency_Memory[i] != 0)
			{
				if (Latency_Memory[i]  < Latency_Minimum )
					Latency_Minimum = Latency_Memory[i] ;
				if (Latency_Memory[i]  > Latency_Maximum )
					Latency_Maximum = Latency_Memory[i] ;
				
				 
				hist[Latency_Memory[i]] = hist[Latency_Memory[i]] + 1;
			}
//						printf("\n");
//						printf("%4d:Latency:%3d clock cycles ",j,Latency_Memory[i]);
		
        }//  if-else
		  
	  } // for j loop


	 for (i = Latency_Minimum; i <= Latency_Maximum; i++)
       {  

		  printf("\nNumber of cases Maximum Latency is equal to %2d parallel clocks is : %4d",i,hist[i]);
		 }
     break; //case 'm'     
		
		
	 
	 case 'z': //Dump register space
	 case 'Z':
		for (j = 0; j < NUMBER_OF_LANES; j++)
		{		
			
					temp = rd_channel((j<<10)+0x100); // From bit 10 it specifies the channel.
					printf("\nChannel %1d Register 0x100 : 0x%2x",j,temp);

					temp = rd_channel((j<<10)+0x101); // From bit 10 it specifies the channel.
					printf("\nChannel %1d Register 0x101 : 0x%2x",j,temp);
				
		for (i = 0x134; i <= 0x139 ; i++)
			{   
					//offset = (j<<10)+i;
					temp = rd_channel((j<<10)+i); // From bit 10 it specifies the channel.
					printf("\nChannel %1d Register 0x%3x : 0x%2x",j,i,temp);
			}	
					temp = rd_channel((j<<10)+7); 
					printf("\nChannel %1d Register 0x7 : 0x%2x",j,temp);	

					temp = rd_channel((j<<10)+0xA); 
					printf("\nChannel %1d Register 0xA : 0x%2x",j,temp);	

					temp = rd_channel((j<<10)+0x110); //
					printf("\nChannel %1d Register 0x110 : 0x%2x",j,temp);		
			
					temp = rd_channel((j<<10)+0x111); //
					printf("\nChannel %1d Register 0x111 : 0x%2x",j,temp);	
	
					temp = rd_channel((j<<10)+0x119); //
					printf("\nChannel %1d Register 0x119 : 0x%2x",j,temp);			
			
		printf("\n");
		}

	
		for (i = 0x100; i <= 0x117 ; i++)
			{   
					temp = rd_pll(i); 
					printf("\nATX PLL Register 0x%3x : 0x%2x",i,temp);
			}	


			
	 break; 		
	
	 case 'y': //Modify CDR settings
	 case 'Y':
		for (j = 0; j < NUMBER_OF_LANES; j++)
		{	
		
		//Set dyn to 31 : 0x1F on the position of 0x7C becomes 0x7C;
		rmw_channel( ((j<<10) + 0x137),0x7C,0x7C);
		
		//Set fix to 3 (0x3).
		rmw_channel( ((j<<10) + 0x136),0x0F,0x03);
		rmw_channel( ((j<<10) + 0x134),0x40,0x00);
		
		//set IPD to 3 (0x18)
		rmw_channel( ((j<<10) + 0x139),0x38,0x18);
	
	//Disable all calibration
	   //rmw_channel( ((j<<10) + 0x100),0xFF,0x00);		
		}

		break;
		
	 case 'p': //Recalibrate ATX PLL
	 case 'P':		
		 
		 recalibrate_pll(ATX_PLL);
		 //printf("\nATX PLL Register 0x101 : 0x%2x",temp);

		
	 break; 	


	 case 'r': //Recalibrate
	 case 'R':

			recalibrate_channel(SelectedChannel,NO_RATE_SWITCH);
	 
	 break; 	
	 
	

//	 case 'q': // Enable non-bonded clocking for TTK to work
//	 case 'Q':		
//		for (j = 0; j < NUMBER_OF_LANES; j++)
//		{	
//		//Set register 0x13F bits [3:0] to serialization factor of 64
//		rmw_channel( ((j<<10) + 0x13F),0x0F,0xE);		
//		//Select serialization factor of 64 	
//      rmw_channel( ((j<<10) + 0x110),0x07,0x03);			
//		//Select Local CGB for working with TTK
//		rmw_channel( ((j<<10) + 0x111),0x1F,0x18);
//		}
//		
//	 break; 

	
		
	 break; 
	  	

	 case 's': //Dump register space of SelectedChannel
	 case 'S':	
		for (i = 0x0; i <= 0x256 ; i++)
			{   
					//offset = (j<<10)+i;
					dump_register_before[i] = rd_channel((SelectedChannel <<10)+i); 
			}	
			printf("\n\nStored all registers from channel %2d in memory\n\n",SelectedChannel);
			
	 break; 
		
	 case 't': //Dump register space of channel 0 after and compare
	 case 'T':	
		for (i = 0x0; i <= 0x256 ; i++)
			{   
					dump_register_after[i] = rd_channel((SelectedChannel <<10)+i); ; 
				   if (dump_register_after[i] != dump_register_before[i])
						printf("\nChannel %2d : Register 0x%3x : Before :0x%2x  After : 0x%2x",SelectedChannel,i,dump_register_before[i],dump_register_after[i]);
			}	
			printf("\n\n");
	 break; 				 

			
			

			
			
		//ODI Measurement on SelectedChannel
			
 
		case 'o':
		case 'O':
		printf("\n");
		printf("\nMeasuring 2D and 1D Eye , this should take around %d seconds .... \n\n",EyeQInterval*9);

	   ///////////////////////////////////////////////////////////////////////				
	   // Call Function to perform Eye measurement on selected channel 
	   ///////////////////////////////////////////////////////////////////////	
		

	   VCCER_Level = VCCER_1030MV; //A10 ES3 Board
		Bandwidth = ABOVE_10GBPS;
	   DFE_Mode = dfe_enable[SelectedChannel];
		

		eye = do_eye_measurement(SelectedChannel, 1000*EyeQInterval, Bandwidth, VCCER_Level, DFE_Mode, BER_Array, ErrorCount_Array
		,&veye, &veye_top,&veye_bottom, &table_phase_start,&eye_phase,&optimum_phase,&Totalbits_ODI,1);
	
			
	   ///////////////////////////////////////////////////////////////////////				
	   // Print Centered 2D Eye
	   ///////////////////////////////////////////////////////////////////////				

		  print_2D_eye (BER_Array,  ErrorCount_Array, &veye_top, &veye_bottom, &table_phase_start);

	   ///////////////////////////////////////////////////////////////////////				
	   // Print Bathtub (1D)
	   ///////////////////////////////////////////////////////////////////////	
		
		  print_1D_eye (BER_Array, &table_phase_start);

	   ///////////////////////////////////////////////////////////////////////				
	   // Print ODI Statistics
	   ///////////////////////////////////////////////////////////////////////	
		
        printf("\nHorizontal Eye Opening Ch %1d: %2d phases",SelectedChannel,eye_phase);
        printf("\nHorizontal Eye Opening Ch %1d: %.2f UI",SelectedChannel,eye);
        printf("\nOptimum Phase Ch %1d         : %2d",SelectedChannel,optimum_phase);
        printf("\nVertical Eye Opening Ch %1d  : %2d steps",SelectedChannel,veye);	
		  printf("\nSampling time per sample     : %2d ms",EyeQInterval);	
		  printf("\nBER Depth                    : %5e",(1/Totalbits_ODI));			
		

		break;	

      case 'x': /* Toggle XOFF */
		case 'X':	
		
		  if (XOFF == 0)
		  {
			  XOFF = 1;
			  printf("XOFF active.\n\n\n");
		  }
		  else
		  {
			  XOFF = 0;
			  printf("XOFF not active.\n\n\n");
		  }

		  Control_Reg = (Control_Reg & (0xF7FF)) | (XOFF << 11); 		  
		  IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);

        usleep(10);

        break; //case 'x'
		  
			
     case '0':
        printf("\nTerminating Program.\n");
        return 0;
       break;
  
      default:
        printf("\n\tPlease enter a value from 0 to Z\n\n\n");
        break;
	
	
		
		
    }
 }
 return 0;
}
